import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;

public class ActionActor  extends Actor
{
    //OBJECT REGULAR DATA
    double myX,myY;             //location of center-of-rotation
    private double imageOffX=0,imageOffY=0; //offset of the center of the image relative to the center of rotation
    boolean hasImageOffset = false;
    double angle; 
    double mass = 1;            
    double CoR = .5;        //coefficient of restitution
    world world;
    protected int actionNumber = 0;
    ArrayList<Double> destX = new ArrayList<Double>();
    ArrayList<Double> destY = new ArrayList<Double>();
    ArrayList<Double> destS = new ArrayList<Double>();
    boolean keep = false;
    
    
    //OBJECT MOVEMENT DATA
    int moveTimer = 0;
    int turnTimer = 0;
    double velx,vely;
    double velang;
    
    //OBJECT MATERIAL DATA
    int material = 0;
    final int NO_MATERIAL = 0;
    final int MATERIAL_METAL = 1;
    final int MATERIAL_PLASTIC = 2;
    final int MATERIAL_WOOD = 3;
    final int MATERIAL_FLESH = 4;
    String[] materialNames = {"null","metal","plastic","crate","flesh"};
    
    //OBJECT SHADOW DATA
    double[] shadowX;
    double[] shadowY;
    int shadowLevel = 0;

    public ActionActor(){
    }
    
    protected void addedToWorld(World w){
        world = (world)w;
    }
    
    
    public int isShadow(){
        return shadowLevel;
    }
    
    public double[] getShadowX(){
        return null;
    }
    
    public double[] getShadowY(){
        return null;
    }
    
    public void act() 
    {
        if(destX.size()!=0){
            seek(destX.get(0),destY.get(0),destS.get(0));
            if(isNear()){
                destX.remove(0);
                destY.remove(0);
                destS.remove(0);
                if(destS.size()==0){
                    actionNumber = 2;
                }
            }
        }else{
            actionNumber = 0;
        }
        if(turnTimer != 0){
            rotateDegrees(velang);
            turnTimer--;
        }
        updateWorldLocation();
    }   
    
    public boolean isNear(){
        return Math.abs(myX-destX.get(0))<.5   &&  Math.abs(myY-destY.get(0))<.5;
    }
    
    public void addDestination(double a,double b,double f){
        destX.add(a);
        destY.add(b);
        destS.add(f);
    }
    
    public void seek(double x,double y,double s){
        double mag = dist(x-myX,y-myY);
        myX += (x-myX)*s/mag;
        myY += (y-myY)*s/mag;
        if(mag<s){
            myX = x;
            myY = y;
        }
    }
    
    protected void move(double dx,double dy){
        setLocation(myX + dx,myY + dy);
    }
    
    protected void move(double dx, double dy , int time){
        addDestination(myX+dx,myY+dy,dist(dx,dy)/time);
    }
    
    public void setLocation(double x,double y){
        myX = x;
        myY = y;
        //super.setLocation((int)(myX-world.tilex),(int)(myY-world.tiley));
    }
    
    public void updateWorldLocation(){
        double imageX;
        double imageY;
        if(hasImageOffset){
            double myCos = Math.cos(angle);
            double mySin = Math.sin(angle);
            imageX = myX + (imageOffX*myCos) - (imageOffY*mySin) - world.tilex;
            imageY = myY + (imageOffY*myCos) + (imageOffX*mySin) - world.tiley;  
        }else{
            imageX = myX - world.tilex;
            imageY = myY - world.tiley;
        }
        super.setLocation((int)imageX,(int)imageY);
        super.setRotation((int)(angle/Math.PI*180));
    }
    
    public void setVelocity(double vx,double vy){
        velx = vx;
        vely = vy;
    }
    
    public void setRotation(double a){
        angle = a;
    }
    
    public void setRotation(int a){
        angle = a;
    }
    
    public void setRotationDegrees(double a){
        angle = a/180.0*Math.PI;
        super.setRotation((int)a);
    }
    
    public void rotate(double a){
        angle += a;
        setRotation(angle);
    }
    
    public void rotateDegrees(double a){
        angle += a/180.0*Math.PI;
        setRotation(angle);
    }
    
    public void rotateTime(double a1,int time){
        turnTimer = time;
        velang = a1/((double)time);
        a1 = a1 + angle;
    }
    
    //******************************************************
    //HELPER METHODD FOR CLEARING IMAGES OF WHITE
    //******************************************************
    
    public void clearImage(){
        for(int i=0;i<getImage().getWidth();i++){
            for(int j=0;j<getImage().getHeight();j++){
                if(getImage().getColorAt(i,j).equals(Color.white)){
                    getImage().setColorAt(i,j,new Color(0,0,0,0));
                }
            }
        }
    }
    
    public void explode(double strength,double angle,double spread,int pieces){
        for(int i=0;i<pieces;i++){
            world.addObject(new shrapnel(this,strength,angle,spread),myX,myY);
        }
        world.removeObject(this);
    
    }
    
    public void setImageOffset(double x,double y){
        imageOffX = x;
        imageOffY = y;
        hasImageOffset = true;
    }
    
    //******************************************************
    //ACTION METHODS: MEANT TO BE OVERWRITEN
    //******************************************************
    
    public boolean action(int a){
        actionNumber = a;
        return false;
    }
    
    public boolean action(int a,double ... inputs){
        actionNumber = a;
        return false;
    }
    
    public int getActionNumber(){
        return actionNumber;
    }
    
    public double dist(double a,double b){
        return Math.sqrt(a*a+b*b);
    }
    
    public double distTo(double a,double b){
        return dist(a-myX,b-myY);
    }
    
    public double distTo(ActionActor b){
        return dist(b.myX - myX,b.myY - myY);
    }
    
    public boolean keep(){
        return keep;
    }
    
    public void hurt(double dx,double dy,double mag){
        if(material == NO_MATERIAL)return;
        Greenfoot.playSound("Materials/" + materialNames[material] + "_hit_" + (Greenfoot.getRandomNumber(2)+1) + ".mp3");
    }
    
    public void setMaterial(int m){
        material = m;
    }
    
    public void setMaterial(String m){
        for(int i=0;i<materialNames.length;i++){
            if(materialNames[i].equals(m)){
                material = i;
            }
        }
    }
    
    public String toString(){
        return super.toString();
    }
    
    public void grab(double x,double y,double ang){
    }
    
    public void release(){
    }
    
}
