import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
import java.awt.Color;



public class Bone  extends ActionActor
{
    List<Bone> childs = new ArrayList<Bone>();
    Bone parent;
    
    //double myX,myY;   //now represents the center of rotation for the image
    
    double localAngle;  //local angle offset
    
    double parentOffsetX;   //represents the offset this objects center-of-rotation from the parents center-of-rotation
    double parentOffsetY;
    
    boolean addedToWorld = false;
    boolean needCenter = false;
    boolean hasParent = false;
    
    public Bone(){
    }
    
    public Bone(Bone p){
        parent = p;
    }
 
    public Bone(Bone p,double offX,double offY){    //create child of another bone
        parent = p;
        setImageOffset(offX,offY);
    }
    
    public Bone(Bone p,double offX,double offY,String image){    //create child of another bone
        parent = p;
        setImageOffset(offX,offY);
        setImage(image);
    }
    
    public Bone(Bone p,double offX,double offY,GreenfootImage image){    //create child of another bone
        parent = p;
        setImageOffset(offX,offY);
        setImage(image);
    }
    
    public Bone(double offX,double offY){           //create master bone
        parent = null;
        setImageOffset(offX,offY);
    }
    
    public Bone(double offX,double offY,String image){           //create master bone
        parent = null;
        setImageOffset(offX,offY);
        setImage(image);
    }
    
    public Bone(double offX,double offY,GreenfootImage image){           //create master bone
        parent = null;
        setImageOffset(offX,offY);
        setImage(image);
    }
    
    protected void addedToWorld(World world){
        super.addedToWorld(world);
        calcOffsets();
        
        if(parent!=null){
            parent.addChild(this);
            parentOffsetX = myX - parent.myX;
            parentOffsetY = myY - parent.myY;  
        }  
        clearImage();
        
        if(parent!=null){
            hasParent = true;
        }else{
            hasParent = false;
        }
    }
    
    public void act(){
        super.act();
        if(hasParent && parent.getWorld()==null){
            getWorld().removeObject(this);
        }
    }
      
    
    protected void move(double a,double b){
        setBoneLocation(myX+a,myY+b);  
    }
    
    public void setBoneLocation(double x,double y){
        for(int i=0;i<childs.size();i++){
            childs.get(i).move(x-myX,y-myY);
        }
        myX = x;
        myY = y;
        calcOffsets();    
    }
    
    public void setBoneLocation(int x,int y){
        setBoneLocation((double)x,(double)y);
    }
    
    public void rotate(double ang){
        localAngle += ang;
        calcOffsets();
    }  
    
    public void rotateDegrees(double ang){
        rotate(ang/180.0*Math.PI);
    }
    
    public void setRotation(double ang){
        if(parent==null){
            setLocalRotation(ang);
        }else{
            setLocalRotation(ang - parent.angle);
        }
    }    
    
    public void setLocalRotation(double ang){
        localAngle = ang;
        calcOffsets();
    }  
    
    public double getLocalRotation(){
        return localAngle;
    }
    
    public void addChild(Bone b){
        childs.add(b);
    }
    
    public void removeChild(Bone b){
        childs.remove(b);
    }
    
    public void removeAllBones(){
        for(int i=0;i<childs.size();i++){
            childs.get(i).removeAllBones();
        }
        world.removeObject(this);
    }
    
    public void calcOffsets(){
        if(parent==null){
            angle = localAngle;
        }else{
            angle = localAngle + parent.angle;
        }
        double cos = Math.cos(angle);
        double sin = Math.sin(angle);
        
        //super.setRotation((int)(180*angle/Math.PI));
        
        Bone tempBone;
        for(int i=0;i<childs.size();i++){
            tempBone = childs.get(i);
            tempBone.calcOffsets();
            tempBone.setBoneLocation(myX + tempBone.parentOffsetX*cos - tempBone.parentOffsetY*sin,
                                     myY + tempBone.parentOffsetY*cos + tempBone.parentOffsetX*sin);
            //tempBone.setLocation(centerX + tempBone.parentOffsetX,centerY + tempBone.parentOffsetY);
        } 
        updateWorldLocation();
    }
    
    public boolean keep(){
        if(parent!=null){
            return parent.keep();
        }else{
            return super.keep();
        }
    }
    
    public List<Bone> getConnectedBones(){
        List<Bone> all = new ArrayList<Bone>();
        for(int i=0;i<childs.size();i++){
            all.addAll(childs.get(i).getConnectedBones());
        }
        all.add(this);
        return all;  
    }
    
    public void disconectAllBones(){
        for(int i=0;i<childs.size();i++){
            childs.get(i).disconectAllBones();
        }
        childs = new ArrayList<Bone>();
        parent = null;
        hasParent = false;
    }
    
    public boolean isChildOf(Bone b){
        if(b == this){
            return true;
        }
        if(!hasParent){
            return false;
        }
        return parent.isChildOf(b);   
    }
    
    public void updateWorldLocation(){
        super.updateWorldLocation();
        for(Bone child : childs){
            child.updateWorldLocation();
        }
    }

}
