import java.util.ArrayList;
import greenfoot.*;  

public class BoneSkeletonController  
{
    ArrayList<BoneLimbController> limbControllers = new ArrayList<BoneLimbController>();
    Bone baseBone;
    int limbNum;
    int resetTimer = 0;
    
    int cycle = 0;
    int cycleLength = 10;
    double cycleThreshold = 0;
    double myOffset;
    int currentPair = 0;
    int cyclePairs = 0;
    int cycleMode = 1; 
        //Mode 0: No cycling
        //Mode 1: Alternating 
        //Mode 2: Symetrical
    public BoneSkeletonController(Bone base,double radius,double offset,int limbs,double armOffX,double armOffY,double handOffX,double handOffY,String armFile,String handFile,world world){
        makeSkeletonController(base,radius,offset,limbs,armOffX,armOffY,handOffX,handOffY,new GreenfootImage(armFile),new GreenfootImage(handFile),world);
    }
    
    public BoneSkeletonController(Bone base,double radius,double offset,int limbs,double armOffX,double armOffY,double handOffX,double handOffY,GreenfootImage armFile,GreenfootImage handFile,world world){
        makeSkeletonController(base,radius,offset,limbs,armOffX,armOffY,handOffX,handOffY,armFile,handFile,world);
    }
    
    
    public void makeSkeletonController(Bone base,double radius,double offset,int limbs,double armOffX,double armOffY,double handOffX,double handOffY,GreenfootImage armImage,GreenfootImage handImage,world world){
        bodyPartBelow arm,hand;
        limbNum = limbs;
        myOffset = offset;
        for(int i=0;i<limbs;i++){
            arm = new bodyPartBelow(base,armOffX,armOffY,armImage);
            world.addObject(arm,base.myX + radius*Math.cos(Math.PI*2*i/limbs + offset),
                                base.myY + radius*Math.sin(Math.PI*2*i/limbs + offset));
            hand = new bodyPartBelow(arm,handOffX,handOffY,handImage);
            world.addObject(hand,arm.myX + armOffX*2,arm.myY + armOffY*2);
                                     
            BoneLimbController controller = new BoneLimbController(base,arm,hand);
            controller.setLengths(Math.sqrt(Math.pow(armOffX*2,2) + Math.pow(armOffY*2,2)),
                                  Math.sqrt(Math.pow(handOffX*2,2) + Math.pow(handOffY*2,2)));
            controller.enableReplanting();
            limbControllers.add(controller); 
            if(i<limbs/2){
                controller.invert();
            }
        }    
        baseBone = base;
        cyclePairs = limbs/2;
    }
    
    public void solve(){
        if(cycle >= cycleLength){
            //System.out.println(cycleMode);
            switch(cycleMode){
                case 0: break ;
                case 1:
                    for(int i=currentPair;i<limbControllers.size();i+=cyclePairs){
                        limbControllers.get(i).resetLimb();
                    } 
                    currentPair++;
                    if(currentPair >= cyclePairs){
                        currentPair = 0;
                    }
                    break;
                case 2:
                    if(currentPair == cyclePairs){
                        currentPair = 0;
                    }else{
                        currentPair = cyclePairs;
                    }
                    for(int i=0;i<cyclePairs;i++){
                        //System.out.println(i+currentPair);
                        limbControllers.get(i+currentPair).resetLimb();
                    }                  
                    break;
                default: break;
            }
            cycle = 0;
        }
        if(Math.sqrt(Math.pow(baseBone.velx,2)+Math.pow(baseBone.vely,2)) > cycleThreshold){
            cycle++;
        }
       
        boolean anyJustPlanted = false;
        for(BoneLimbController controller : limbControllers){
            controller.solve();
            anyJustPlanted |= controller.justPlanted;
        }
        if(anyJustPlanted){
            baseBone.action(20);
        }
        if(resetTimer > 0){
            resetTimer--;
            if(resetTimer==1){
                for(int i=0;i<limbControllers.size();i+=2){
                    limbControllers.get(i).resetLimb();
                }  
            }
        }      
    }
    
    public void resetLimbs(){
        if(resetTimer>0){
            return;
        }
        for(int i=1;i<limbControllers.size();i+=2){
            limbControllers.get(i).resetLimb();
        }  
        resetTimer = 30;
    }
    
    public BoneLimbController getLimbController(int i){
        return limbControllers.get(i);
    }

    public void setAngleConstant(double a){
        for(BoneLimbController controller : limbControllers){
            controller.setAngleConstant(a);
        }
    }
    
    public void drift(double a,double b){
        for(BoneLimbController controller : limbControllers){
            controller.plantX += a;
            controller.plantY += b;
        }
    }
    
    public void setCycleLength(int length){
        cycleLength = length;
    }
    
    public void setCycleThreshold(double threshold){
        cycleThreshold = threshold;
    }
    
    public void setCycleMode(int m){
        cycleMode = m;
    }
    
}