import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;


public class Cable  extends ActionActor
{
    double[] lx;
    double[] ly;
    double[] lvx;
    double[] lvy;
    double[] l;
    int subCurves = 2;
    Cable slave;
    Cable master;
    double slavex,slavey;
    boolean isMaster = false;
    final double friction = .9;
    final double elasticity = 4;
    final double segmentLength = 10;
    final double maxLength;
    
    boolean masterGrabbed = false;
    boolean slaveGrabbed = false;
    
    Connector masterConnection;
    Connector slaveConnection;
    boolean masterConnected = false;
    boolean slaveConnected = false;
    
    boolean locked = false;
    boolean addedToWorld = false;
    double grabbedDistance;
    double[] binomial;
    
    //int links does not include endpoint, and is number of point connections between the cable ends
    //links = 1 would include two cable ends and a single conneciton point
    //each connection from link to link is 5 pixels
    public Cable(int links,double sx,double sy){
        lx = new double[links+2];
        ly = new double[links+2];
        lvx = new double[links+2];
        lvy = new double[links+2];
        l = new double[links+1];
        for(int i=0;i<l.length;i++){
            l[i] = 25;
        }
        slavex = sx;
        slavey = sy;
        isMaster = true;
        slave = new Cable();
        slave.master = this;
        maxLength = segmentLength*(links+3);
        binomial = new double[lx.length];
        for(int i=0;i<lx.length;i++){
            binomial[i] = binomial(lx.length-1,i);
        }
    }
    
    public Cable(){
        binomial = null;
        isMaster = false;
        maxLength = 0;
    }
    
    protected void addedToWorld(World world){
        if(addedToWorld){
            return;
        }
        super.addedToWorld(world);
        addedToWorld = true;
        if(isMaster){
            world.addObject(slave,(int)slavex,(int)slavey);
            for(double i=0;i<lx.length;i++){
                lx[(int)i] = slavex*(i/lx.length) + myX*((lx.length-i)/lx.length) + Math.random();
                ly[(int)i] = slavey*(i/ly.length) + myY*((ly.length-i)/ly.length) + Math.random();
            }
        }
        clearImage();
    }
    
    
    public void act() 
    {
        if(!isMaster){
            super.act();
            return;
        }
        double totalLength = 0;
        for(int j=0;j<8;j++){
            totalLength = 0;
            for(int i=0;i<l.length;i++){
                double length = Math.sqrt(Math.pow(lx[i+1]-lx[i],2) + Math.pow(ly[i+1]-ly[i],2));
                totalLength += length;
            
                lvx[i] += (lx[i+1] - lx[i])/length*(length-segmentLength)/elasticity;
                lvy[i] += (ly[i+1] - ly[i])/length*(length-segmentLength)/elasticity;
                lvx[i+1] += (lx[i] - lx[i+1])/length*(length-segmentLength)/elasticity;
                lvy[i+1] += (ly[i] - ly[i+1])/length*(length-segmentLength)/elasticity;
            

                //world.drawLine(lx[i],ly[i],lx[i+1],ly[i+1],2);         
            }
            for(int i=0;i<lx.length;i++){ 
                lvx[i] *= friction;
                lvy[i] *= friction;
                lx[i] += lvx[i];
                if(world.tileSolid(lx[i],ly[i])){
                    lx[i] -= lvx[i];
                    lvx[i] = 0;    
                }
                ly[i] += lvy[i];
                if(world.tileSolid(lx[i],ly[i])){
                    ly[i] -= lvy[i];
                    lvy[i] = 0; 
                }
            }  
        }
        
        if(!masterGrabbed){
            List<Connector> cons = getIntersectingObjects(Connector.class);
            for(int i=0;i<cons.size();i++){
                Connector c = cons.get(i);
                if(!c.isConnected()){
                    masterConnected = true;
                    masterConnection = c;
                    masterConnection.connect(this);
                }
            }
        }
        
        if(!slaveGrabbed){
            List<Connector> cons = slave.getIntersectingObjects(Connector.class);
            for(int i=0;i<cons.size();i++){
                Connector c = cons.get(i);
                if(!c.isConnected()){
                    slaveConnected = true;
                    slaveConnection = c;
                    slaveConnection.connect(slave);
                    world.character.grabbing = false;
                    slaveGrabbed = false;
                }
            }
        }
        if(totalLength > maxLength*1.2  ||  grabbedDistance>55){
            slaveGrabbed = false;
            masterGrabbed = false;
            world.character.grabbing = false;    
            grabbedDistance = 0;
        }
        if(masterConnected){
            lx[0] = masterConnection.myX;
            ly[0] = masterConnection.myY;
            lx[1] = masterConnection.myX + masterConnection.dx*segmentLength;
            ly[1] = masterConnection.myY + masterConnection.dy*segmentLength;
            if(totalLength > maxLength){
                slaveGrabbed = false;
                masterGrabbed = false;
                world.character.grabbing = false;    
            }
        }
        if(slaveConnected){
            lx[lx.length-1] = slaveConnection.myX;
            ly[ly.length-1] = slaveConnection.myY;
            lx[lx.length-2] = slaveConnection.myX + slaveConnection.dx*segmentLength;
            ly[ly.length-2] = slaveConnection.myY + slaveConnection.dy*segmentLength;
            if(totalLength > maxLength){
                slaveGrabbed = false;
                masterGrabbed = false;
                world.character.grabbing = false;    
            }
        }
        
        setLocation(lx[0],ly[0]);
        setRotation(Math.atan2(ly[0]-ly[1],lx[0]-lx[1]));
        slave.setLocation(lx[lx.length-1],ly[ly.length-1]);
        slave.setRotation(Math.atan2(ly[lx.length-1]-ly[lx.length-2],lx[lx.length-1]-lx[lx.length-2]));
        
        double[] beizerQuality = {.5,.2,.1,.05};
        beizer(lx,ly,beizerQuality[world.curveQuality]);
        super.act();
    }    
    
    public boolean action(int a){
        switch(a){
            case 1:grabbed();return true;
            case 4:lock();return true;
            case 3:masterConnection.disconnect();masterConnected = false;return true;
            case 2:master.slaveConnection.disconnect();master.slaveConnected = false;return true;
        }
        return false;
    }
    
    public void grabbed(){
        if(isMaster){
            if(locked){
                return;
            }
            masterGrabbed = true;
            if(masterConnected){
                masterConnection.disconnect();
                masterConnected = false;
            }
            world.character.grabbed(this);
        }else{
            if(master.locked){
                return;
            }
            master.slaveGrabbed = true;
            if(master.slaveConnected){
                master.slaveConnection.disconnect();
                master.slaveConnected = false;
            }
            world.character.grabbed(master);
        }
        
    }
    
    public void grab(double x,double y,double ang){
        if(masterGrabbed){
            lvx[0] += (x - lx[0])/elasticity;
            lvy[0] += (y - ly[0])/elasticity;
            grabbedDistance = world.character.distTo(lx[0],ly[0]);
        }
        if(slaveGrabbed){
            lvx[lvx.length-1] += (x - lx[lvx.length-1])/elasticity;
            lvy[lvx.length-1] += (y - ly[lvx.length-1])/elasticity;
            grabbedDistance = world.character.distTo(lx[lx.length-1],ly[ly.length-1]);
        }
    }
    
    public void release(){
        masterGrabbed = false;
        slaveGrabbed = false;
    }
   
    public void beizer(double[] xr, double[] yr,double dt){
        double px=0,py=0;
        boolean drawn = false;
        for(double t=0;t<=1.01;t+=dt){
            double xt = 0;
            double yt = 0;
            for(int i=0;i<xr.length;i++){
                double cons = binomial[i]*pow(1-t,xr.length-1-i)*pow(t,i);
                xt += cons*xr[i];
                yt += cons*yr[i];
            }
            if(drawn){
                world.drawLine(xt,yt,px,py,2);
            }
            drawn = true;
            px = xt;
            py = yt;
        }
        
    }
            
    public double pow(double d,int p){
        if(p==0){
            return 1;
        }
        return Math.pow(d,p);
    }
    
    public double binomial(int n,int k){
        if(n==0){
            return 0;
        }
        if(k==0){
            return 1;
        }
        return factorial(n)/(factorial(k)*factorial(n-k));
    }
    
    public double factorial(int n){
        double f=1;
        for(int i=1;i<=n;i++){
            f *= i;
        }
        return f;
    }
    
    public void lock(){
        //System.out.println("locked: " + this);
        if(isMaster){
            slave.locked = true;
        }else{
            master.locked = true;
        }
        locked = true;
    }
    
}
