import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.StringTokenizer;




public class EventScript  extends Actor
{
    final boolean debug = false;
    boolean addedToWorld = false;
    final String[] commandList = {"TurnOnAss",
                                  "TurnOffAss",
                                  "LockFlashlight",
                                  "TurnOnFlashlight",
                                  "TurnOffFlashlight",
                                  "FlickerFlashlight",
                                  "PlaySound",
                                  "StartScript",
                                  "LoadScript",
                                  "Terminate",
                                  "Pause",
                                  "Sleep",
                                  "SetTint",
                                  "SetTile",
                                  "SetSolid",
                                  "GiveFlashlight",
                                  "Say",
                                  "Transition",
                                  "SetDarkness",
                                  "SetLightShadow",
                                  "TurnOnShadows",
                                  "TurnOffShadows",
                                  "SetShake",
                                  "Spark",
                                  "WakeTrigger",
                                  "EndTrigger",
                                  "GiveWeapon",
                                  "DisplayLevelName",
                                  "PlayerForce",
                                  "SetPlayerLocation",
                                  "MovePlayer",
                                  "CreateWeb",};
    final String[] actionList = {"AddToWorld", 
                                 "RemoveFromWorld", 
                                 "Move", 
                                 "Rotate",
                                 "AddDestination",
                                 "Keep",
                                 "RotateTime",
                                 "SetShadowLevel",
                                 "Explode",
                                 "setLocation",
                                 "setMaterial",
                                 "Use",
                                 "CloseFast",
                                 "OpenFast",
                                 "Lock",
                                 "Unlock",
                                 "Open",
                                 "Close",                     
                                 "OpenAway",
                                 "OpenAwayFast",
                                 
                                 "Play",
                                 "Pause",
                                 "Stop",
                                 "PlayLoop",
                                 
                                 "giveFlashlight",
                                 "flickerLight",
                                 "flickerMedium",
                                 "flickerHeavy",
                                 "flickerOff",
                                 "flicker",
                                 "turnOffShadows",
                                 "turnOnShadows",
                                 "setFlickerSplit",
                                 "setUpdateFrequency",
                                 "setBrightness",
                                 "setFlickerCenter",
                                
                                 "setVolume",
                                 "fade",
                                 "smoothLoop",
                                
                                 "targetPerson",
                                 "wakeClaw",
                                 "chomp",
                                 "break",};
    final int[] actionEquates = {0, 
                                 1,   
                                 2, 
                                 3,
                                 4,
                                 5,
                                 6,
                                 7,
                                 8,
                                 9,
                                 10,
                                 11,
                                 12,
                                 13,
                                 14,
                                 15,
                                 16,
                                 17,
                                 18,
                                 19,
                                 
                                 11,
                                 12,
                                 13,
                                 14,
                                 
                                 12,
                                 12,
                                 13,
                                 14,
                                 15,
                                 21,
                                 16,
                                 17,
                                 22,
                                 23,
                                 24,
                                 25,
                                
                                 20,
                                 31,
                                 22,
                                
                                 11,
                                 12,
                                 12,
                                 13,};
    ArrayList<String> renderedStrings = new ArrayList<String>();
    ArrayList<Double> commands = new ArrayList<Double>();
    ArrayList<Actor> declaredObjects = new ArrayList<Actor>();
    ArrayList<String> objectEquates = new ArrayList<String>();
    int address = 0;
    boolean asleep = false;
    int timer = 0;
    world world;
    
    boolean error = false;

    public EventScript(){
    }
    
    public String next(Scanner line,String argument){
        if(!line.hasNext()){
            System.out.println("ERROR: Missing " + argument + " on line");
            error = true;
            return "0";
        }
        return line.next();
    }
    
    public EventScript(String file,world world){
        if(debug)System.out.println("Loading Script file: " + file);
        ArrayList<String> commandLines = new ArrayList<String>();
        try{
            URL path = getClass().getClassLoader().getResource("Scripts/" + file);
            InputStream input = path.openStream();
            BufferedReader buffer = new BufferedReader(new InputStreamReader(input));
            String line = buffer.readLine(); 
            while(line != null)
            {
                if (line != null)
                commandLines.add(line);
                line = buffer.readLine();
            }
            input.close();
            buffer.close();
        }catch (Exception e){
            System.out.println("Error: " + e);
            System.out.println("Could not find/open " + file);
            error = true;return;
        }  
        if(debug)System.out.println("File opened");
        
        
        for(int i=0;i<commandLines.size();i++){
            Scanner line = new Scanner(commandLines.get(i));
            if(line.hasNext()){
                String startCommand = line.next().toLowerCase();          
                
                if(startCommand.equals("#actionactor")){
                    if(debug)System.out.println("parsing ActionActor");
                    String actorName = next(line,"object types");
                    boolean global = false;
                    if(actorName.equals("#Global")){
                        global = true;
                        actorName = next(line,"object name");
                        if(debug)System.out.println("    Global variable initilized");
                    }
                    if(actorName.equals("@Global")){
                        String name = next(line,"object name");
                        if(debug)System.out.println("Equate: "+name+" added from global variable");
                        objectEquates.add(name);
                        declaredObjects.add(world.declaredObjects.get(world.objectEquates.indexOf(name)));
                    }
                    if(actorName.equals("StaticImage")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new StaticImage(new GreenfootImage(next(line,"file name"))));
                    }
                    if(actorName.equals("StaticOverlay")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new StaticOverlay(new GreenfootImage(next(line,"file name"))));
                    }
                    if(actorName.equals("Medpack")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Medpack());
                    }
                    if(actorName.equals("Ammo")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Ammo(Integer.parseInt(next(line,"Ammo Amount")),
                                                     Integer.parseInt(next(line,"Ammo Type"))));
                    }
                    if(actorName.equals("Flare")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Flare());
                    }
                    if(actorName.equals("FanShadow")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new AnimatedShadow(Integer.parseInt(next(line,"Spoke Number")),
                                                               Integer.parseInt(next(line,"Between Number")),
                                                               Double.parseDouble(next(line,"Spoke Angle")),
                                                               Double.parseDouble(next(line,"Fan Speed")),
                                                               Double.parseDouble(next(line,"Fan Small Radius")),
                                                               Double.parseDouble(next(line,"Fan Large Radius")),
                                                               Float.parseFloat(next(line,"Shadow Brightness")),
                                                               (StaticLight)declaredObjects.get(objectEquates.indexOf(next(line,"light name")))));
                    }
                    if(actorName.equals("Lever")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Lever());
                    }
                    if(actorName.equals("StaticLight")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new StaticLight(Integer.parseInt(next(line,"Light size")),Float.parseFloat(next(line,"Light brightness"))));
                    }
                    if(actorName.equals("BeamLight")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new StaticLight(Integer.parseInt(next(line,"Light size")),
                                                            Float.parseFloat(next(line,"Light brightness")),
                                                            Double.parseDouble(next(line,"Beam Angle")),
                                                            Double.parseDouble(next(line,"Beam Width")),
                                                            Double.parseDouble(next(line,"Beam Caper"))));
                    }
                    if(actorName.equals("BoundedBeamLight")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new StaticLight(Integer.parseInt(next(line,"Light size")),
                                                            Float.parseFloat(next(line,"Light brightness")),
                                                            Double.parseDouble(next(line,"Beam Angle")),
                                                            Double.parseDouble(next(line,"Beam Width")),
                                                            Double.parseDouble(next(line,"Beam Caper")),
                                                            Integer.parseInt(next(line,"Left Side")),
                                                            Integer.parseInt(next(line,"Right Side")),
                                                            Integer.parseInt(next(line,"Top Side")),
                                                            Integer.parseInt(next(line,"Bottom Side"))));
                    }
                    if(actorName.equals("SolidLight")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new StaticLight(Integer.parseInt(next(line,"Light width")),
                                                            Integer.parseInt(next(line,"Light height")),
                                                            Integer.parseInt(next(line,"Light center X")),
                                                            Integer.parseInt(next(line,"Light center Y")),
                                                            Float.parseFloat(next(line,"Light brightness"))));
                    }
                    if(actorName.equals("BoundedLight")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new StaticLight(Integer.parseInt(next(line,"Light size")),
                                                            Float.parseFloat(next(line,"Light brightness")),
                                                            Integer.parseInt(next(line,"Light width")),
                                                            Integer.parseInt(next(line,"Light height")),
                                                            Integer.parseInt(next(line,"Light centerX")),
                                                            Integer.parseInt(next(line,"Light centerY"))));
                    }
                    if(actorName.equals("Crate")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Crate(Integer.parseInt(next(line,"Crate size"))));
                    }
                    if(actorName.equals("Grim")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Grim());
                    }
                    if(actorName.equals("GlowingEyes")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new GlowingEyes());
                    }
                    if(actorName.equals("RadioFlasher")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new RadioFlasher());
                    }
                    if(actorName.equals("Person")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Person(next(line,"Person Type")));
                    }
                    if(actorName.equals("Zombie")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Zombie(next(line,"Zombie Type")));
                    }
                    if(actorName.equals("MiniSpider")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new MiniSpider(!next(line,"Wandering").equals("Wandering")));
                    }
                    if(actorName.equals("Spider")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Spider());
                    }
                    if(actorName.equals("MrFriendly")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new MrFriendly());
                    }
                    if(actorName.equals("Dave")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Dave());
                    }
                    if(actorName.equals("Claw")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Claw(next(line,"Current State").equals("sleeping")));
                    }
                    if(actorName.equals("Cable")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Cable(Integer.parseInt(next(line,"Cable Segments")),Integer.parseInt(next(line,"PositionX")),Integer.parseInt(next(line,"PositionY"))));
                    }
                    if(actorName.equals("Connector")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Connector(Integer.parseInt(next(line,"PositionX")),Integer.parseInt(next(line,"PositionY")),Integer.parseInt(next(line,"Angle")),Integer.parseInt(next(line,"Angle"))));
                    }
                    if(actorName.equals("Sound")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        String filename = next(line,"file name");
                        if(line.hasNext()){
                            declaredObjects.add(new Sound(Double.parseDouble(next(line,"Sound Radius")),filename));
                        }else{
                            declaredObjects.add(new Sound(filename));
                        }
                    }
                    if(actorName.equals("WorldSound")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new Sound(next(line,"file name"),Integer.parseInt(next(line,"Sound Radius"))));
                    }
                    if(actorName.equals("Door")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        boolean locked = next(line,"argument").equals("locked");
                        boolean open = next(line,"argument").equals("open");
                        int closedAngle = Integer.parseInt(next(line,"argument"));
                        declaredObjects.add(new Door(locked,open,closedAngle));
                    }
                    if(actorName.equals("BloodSlide")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new BloodSlide(Integer.parseInt(next(line,"blood length")),
                                                           Integer.parseInt(next(line,"blood width")),
                                                           Integer.parseInt(next(line,"blood angle")),
                                                           Integer.parseInt(next(line,"blood coeficient"))));
                    }
                    if(actorName.equals("BloodSmears")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        declaredObjects.add(new BloodSmears(Integer.parseInt(next(line,"blood length")),
                                                           Integer.parseInt(next(line,"blood width")),
                                                           Integer.parseInt(next(line,"blood smears")),
                                                           Integer.parseInt(next(line,"blood coeficient")),
                                                           Integer.parseInt(next(line,"smear lengths"))));
                    }
                    if(actorName.equals("DoubleDoor")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        boolean locked = next(line,"argument").equals("locked");
                        boolean open = next(line,"argument").equals("open");
                        int closedAngle = Integer.parseInt(next(line,"argument"));
                        Door newDoor = new Door(locked,open,closedAngle);
                        Door slaveDoor = new Door(newDoor);
                        declaredObjects.add(newDoor);
                    }
                    if(actorName.equals("Shadow")){
                        objectEquates.add(next(line,"object name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        int shadowType = Integer.parseInt(next(line,"shadowType"));
                        ArrayList<Integer> buffer = new ArrayList<Integer>();
                        while(line.hasNext()){
                            buffer.add(Integer.parseInt(next(line,"shadowPositionArgument")));
                        }
                        double[] shax = new double[buffer.size()/2];
                        double[] shay = new double[buffer.size()/2];
                        for(int k=0;k<shax.length;k++){
                            shax[k] = buffer.get(k);
                            shay[k] = buffer.get(k+shax.length);
                        }
                        Shadow shadow = new Shadow(shax,shay,shadowType);
                        declaredObjects.add(shadow);
                    }
                    if(global){
                        world.declaredObjects.add(declaredObjects.get(declaredObjects.size()-1));
                        world.objectEquates.add(objectEquates.get(objectEquates.size()-1));      
                    }
                }else if(startCommand.equals("#trigger")){
                    if(debug)System.out.println("parsing Trigger");
                    String objectName = next(line,"Trigger Type");
                    boolean sleeping = false;
                    boolean sustaining = false;
                    boolean edge = false;
                    if(objectName.equals("#Sleeping")){
                        sleeping = true;
                        objectName = next(line,"trigger name");
                        if(debug)System.out.println("    Sleeping Trigger initilized");
                    }
                    if(objectName.equals("#Sustaining")){
                        sustaining = true;
                        objectName = next(line,"trigger name");
                        if(debug)System.out.println("    Sustaining Trigger initilized");
                    }  
                    if(objectName.equals("#Sleeping")){
                        sleeping = true;
                        objectName = next(line,"trigger name");
                        if(debug)System.out.println("    Sleeping Trigger initilized");
                    }
                    if(objectName.equals("#Edge")){
                        edge = true;
                        objectName = next(line,"trigger name");
                        if(debug)System.out.println("    Edge Trigger initilized");
                    }  
                    if(objectName.equals("TriggerSelection")){
                        objectEquates.add(next(line,"Trigger name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        TriggerSelection currentTrigger = new TriggerSelection(Integer.parseInt(next(line,"Selection Numbe")),this);
                        declaredObjects.add(currentTrigger);
                        while(line.hasNext()){
                            currentTrigger.addTrigger((Trigger)declaredObjects.get(objectEquates.indexOf(next(line,"Trigger Equate"))));
                        }
                    } 
                    if(objectName.equals("TriggerTimer")){
                        objectEquates.add(next(line,"Trigger name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        int x1 = Integer.parseInt(next(line,"Trigger Time"));
                        int x2 = Integer.parseInt(next(line,"Trigger Offset"));
                        declaredObjects.add(new TriggerTimer(x1,x2,this));
                    } 
                    if(objectName.equals("TriggerTile")){
                        objectEquates.add(next(line,"Trigger name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        int x1 = Integer.parseInt(next(line,"argument"));
                        int x2 = Integer.parseInt(next(line,"argument"));
                        declaredObjects.add(new TriggerTile(x1,x2,this));
                    } 
                    if(objectName.equals("TriggerWatch")){
                        objectEquates.add(next(line,"Trigger name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        ActionActor watcher = (ActionActor)declaredObjects.get(objectEquates.indexOf(next(line,"watcher name")));
                        int action = Integer.parseInt(next(line,"argument name"));
                        declaredObjects.add(new TriggerWatch(watcher,action,this));
                    } 
                    if(objectName.equals("TriggerRect")){
                        objectEquates.add(next(line,"Trigger name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        int x1 = Integer.parseInt(next(line,"argument"));
                        int y1 = Integer.parseInt(next(line,"argument"));
                        int x2 = Integer.parseInt(next(line,"argument"));
                        int y2 = Integer.parseInt(next(line,"argument"));
                        declaredObjects.add(new TriggerRect(x1,y1,x2,y2,this));
                    }  
                    if(objectName.equals("TriggerLook")){
                        objectEquates.add(next(line,"Trigger name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");    
                        int x = Integer.parseInt(next(line,"argument"));
                        int y = Integer.parseInt(next(line,"argument"));
                        int r = Integer.parseInt(next(line,"argument"));
                        declaredObjects.add(new TriggerLook(x,y,r,this));
                    }   
                    if(objectName.equals("TriggerAND")){
                        objectEquates.add(next(line,"Trigger name"));
                        if(debug)System.out.println("Equate: "+objectEquates.get(objectEquates.size()-1)+" added");
                        TriggerAND newAnd = new TriggerAND(this);
                        boolean reparent = true;
                        while(line.hasNext()){
                            
                            String argument = line.next();
                            if(debug)System.out.println("     "+argument+" added to AND Trigger");
                            
                            if(argument.equals("disableReparent")){
                                reparent = false;
                                argument = line.next();
                            }
                            
                            int index = objectEquates.indexOf(argument);
                            if(index==-1){
                                System.out.println("ERROR: Bad name on line");
                                error = true;index = 0;
                            }
                            newAnd.addChild(((Trigger)declaredObjects.get(index)));
                            if(reparent){
                                ((Trigger)declaredObjects.get(index)).setParent(newAnd);
                            }
                        }
                        //newAnd.setChilds(childs);
                        declaredObjects.add(newAnd);
                        //declaredObjects.add(new TriggerLook(Integer.parseInt(line.next()),Integer.parseInt(line.next()),Integer.parseInt(line.next()),this));
                    }
                    if(sustaining){
                        ((Trigger)declaredObjects.get(declaredObjects.size()-1)).sustain();
                    }
                    if(sleeping){
                        ((Trigger)declaredObjects.get(declaredObjects.size()-1)).sleep();
                    }
                    if(edge){
                        ((Trigger)declaredObjects.get(declaredObjects.size()-1)).setEdge(true);
                    }
                }else if(startCommand.equals("if")){
                    String equate = next(line,"Trigger name");
                    
                    int index = objectEquates.indexOf(equate);
                    if(index==-1){
                        System.out.println("ERROR: Bad name on line");
                        error = true;index = 0;
                    }else{
                        if(debug)System.out.println("Entrance into equate: " + equate + " at index: " + commands.size());
                        ((Trigger)declaredObjects.get(index)).setJumpAddress(commands.size());
                    }
                } else if(!startCommand.equals("//")){
                    if(debug)System.out.println(":"+startCommand);
                    if(objectEquates.indexOf(startCommand)!=-1){
                        commands.add(99.0);
                        int index = objectEquates.indexOf(startCommand);
                        if(index==-1){
                            System.out.println("ERROR: Bad command on line");
                            error = true;index = 0;
                        }
                        commands.add((double)index);
                    }else{
                        int indexC = getCommand(startCommand);
                        if(indexC==-1){
                            System.out.println("ERROR: Bad command on line");
                            error = true;indexC = 0;
                        }
                        commands.add((double)indexC);
                    }
                    
                    while(line.hasNext()){
                         
                        String data = line.next();
                        
                        if(isNumber(data)){
                            commands.add(Double.parseDouble(data));
                        }else if(getAction(data)!=-1){
                            commands.add((double)getAction(data)); 
                        }else if(objectEquates.indexOf(data)!=-1){
                            commands.add((double)objectEquates.indexOf(data));  
                        }else{
                            if(renderedStrings.indexOf(data)!=-1){
                                commands.add((double)renderedStrings.indexOf(data));
                            }else{
                                commands.add((double)renderedStrings.size());
                                renderedStrings.add(data);
                            }
                        }
                    }
                }
                if(error){
                    System.out.println("        " + commandLines.get(i));
                    return;
                }
            }
        }  
        if(debug)System.out.println("Event Script: " + file + " parsed and created successfully");
        if(debug)System.out.println(commands);
        if(debug)System.out.println(renderedStrings);

    }
    
    protected void addedToWorld(World w){
        if(addedToWorld){
            return;
        }
        addedToWorld = true;
        for(int i=0;i<declaredObjects.size();i++){
            if(!(declaredObjects.get(i) instanceof ActionActor)){
                getWorld().addObject(declaredObjects.get(i),0,0);
                if(debug)System.out.println("Object: " + declaredObjects.get(i) + " added to world");
            }else{
                if(debug)System.out.println("Object: " + declaredObjects.get(i) + " initialized");
            }
        }
        world = (world)w;
        if(debug)System.out.println();
        //world.addObject(new TriggerLook(500,340,2,this),0,0);
    }
    
    public void act() 
    {
        while(!asleep){
            excecuteCommand();
            if(timer!=0){
                break;
            } 
        }
        
    }    
    
    public boolean trigger(int jumpAddress){
        if(!asleep){
            if(debug)System.out.println("Trigger ignored");
            if(debug)System.out.println();
            return false;
        }
        asleep = false;
        address = jumpAddress;
        if(debug)System.out.println("Excecution resumed");
        if(debug)System.out.println("Excecution jump to address: " + jumpAddress);
        if(debug)System.out.println();
        return true;
    }
    
    public boolean isNumber(String number){
        try{
            Double.parseDouble(number);
            return true;
        }catch (NumberFormatException e){
            return false;
        }
    }
    
    public int getCommand(String input){
        for(int i=0;i<commandList.length;i++){
            if((input.toLowerCase()).equals(commandList[i].toLowerCase())){
                return i;
            }
        }
        return -1;
    }
    
    public int getAction(String input){
        for(int i=0;i<actionList.length;i++){
            if((input.toLowerCase()).equals(actionList[i].toLowerCase())){
                return actionEquates[i];
            }
        }
        return -1;
    }
    
    public void excecuteCommand(){
        double commandD = getIntCommand(address);
        int command = (int)commandD;
        if(debug){
            if(command == 10){
                if(timer == 0){
                    System.out.println();
                }
            }else{
                if(command!=99){
                    System.out.print(":"+command);
                }
            }
        }
        address++;
        switch(command){
            case 0: turnOnAss();break;
            case 1: turnOffAss();break;
            case 2: lockFlashlight();break;
            case 3: turnOnFlashlight();break;
            case 4: turnOffFlashlight();break;
            case 5: flickerFlashlight();break;
            case 6: playSound();break;
            case 7: startScript();break;
            case 8: loadScript();break;
            case 9: terminate();break;
            case 10: pause();break;
            case 11: sleep();break;
            case 12: setTint();break;
            case 13: setTile();break;
            case 14: setSolid();break;
            case 15: giveFlashlight();break;
            case 16: say();break;
            case 17: transition();break;
            case 18: setDarkness();break;
            case 19: setLightShadow();break;
            case 20: turnOnShadows();break;
            case 21: turnOffShadows();break;
            case 22: setShake();break;
            case 23: spark();break;
            case 24: wakeTrigger();break;
            case 25: endTrigger();break;
            case 26: giveWeapon();break;
            case 27: displayLevelName();break;
            case 28: playerForce();break;
            case 29: setPlayerLocation();break;
            case 30: movePlayer();break;
            case 31: createWeb();break;
            case 99: ActionObject();break;
            default: return;
        }
        return;
    }
    
    public void ActionObject(){
        double commandD = getIntCommand(address);
        ActionActor action1 = (ActionActor)declaredObjects.get((int)commandD);
        address++;
        commandD = getIntCommand(address);
        int command = (int)commandD;
        address++;
        
        if(command >=20 && command<30){
            action1.action(command-20,commands.get(address));
            address++;
            return;
        }
        if(command >=30 && command<40){
            action1.action(command-30,commands.get(address),commands.get(address+1));
            address += 2;
            return;
        }
        if(command >=40 && command<50){
            action1.action(command-40,commands.get(address),commands.get(address+1),commands.get(address+2));
            address += 3;
            return;
        }
        
        if(debug)System.out.print("#"+command);
        switch(command){
            case 0: ActionAddToWorld(action1);break;
            case 1: ActionRemoveFromWorld(action1);break;
            case 2: ActionMove(action1);break;
            case 3: ActionRotate(action1);break;
            case 4: ActionAddDestination(action1);break;
            case 5: ActionKeep(action1);break;
            case 6: ActionRotateTime(action1);break;
            case 7: ActionSetShadowLevel(action1);break;
            case 8: ActionExplode(action1);break;
            case 9: ActionSetLocation(action1);break;
            case 10: ActionSetMaterial(action1);break;
            default: action1.action(command-10);
        }
        return;
    }
    
    public void ActionSetMaterial(ActionActor action){
        action.setMaterial(renderedStrings.get(getIntCommand(address)));
        address += 1;  
    }
    
    public void ActionSetLocation(ActionActor action){
        action.setLocation(commands.get(address),commands.get(address+1));
        address += 2;  
    }
    
    public void ActionExplode(ActionActor action){
        action.explode(getIntCommand(address),getIntCommand(address+1)/180.0*Math.PI,getIntCommand(address+2),getIntCommand(address+3));
        address += 4;
    }
    
    public void ActionSetShadowLevel(ActionActor action){
        action.shadowLevel = getIntCommand(address);
        address++;
    }
    
    public void ActionRotateTime(ActionActor action){
        action.rotateTime(getIntCommand(address),getIntCommand(address+1));
        address += 2;
    }
    
    public void ActionRotate(ActionActor action){
        action.rotateDegrees(getIntCommand(address));
        address += 1;
    }
    
    public void ActionMove(ActionActor action){
        action.move(getIntCommand(address),getIntCommand(address+1),getIntCommand(address+2));
        address += 3;
    }
    
    public void ActionAddToWorld(ActionActor action){
        world.addObject(action,commands.get(address),commands.get(address+1));
        address += 2;
    }
    
    public void ActionRemoveFromWorld(ActionActor action){
        getWorld().removeObject(action);
        if(action instanceof Door){
            if(((Door)action).friend != null){
                getWorld().removeObject(((Door)action).friend);
            }
        }
    }
    
    public void ActionKeep(ActionActor action){
        action.keep = true;
    }
    
    public void ActionAddDestination(ActionActor action){
        action.addDestination(commands.get(address),commands.get(address+1),commands.get(address+2));
        address += 3;
    }
    
    public void createWeb(){
        Web.createWeb(world,commands.get(address),commands.get(address+1),getIntCommand(address+2));
        address += 3;
    }
    
    public void movePlayer(){
        world.character.setBoneLocation(world.character.myX+commands.get(address),world.character.myY+commands.get(address+1));
        world.tilex = (int)world.character.myX - 200;
        world.tiley = (int)world.character.myY - 200;
        world.character.updateWorldLocation();
        world.render();
        address += 2;
    }
    
    public void setPlayerLocation(){
        world.character.setBoneLocation(commands.get(address),commands.get(address+1));
        world.character.updateWorldLocation();
        world.tilex = (int)world.character.myX - 200;
        world.tiley = (int)world.character.myY - 200;
        world.render();
        address += 2;
    }
    
    public void displayLevelName(){
        world.displayLevelName();
    }
    
    public void setTile(){
        int tx = getIntCommand(address);
        int ty = getIntCommand(address+1);
        int t = getIntCommand(address+2);
        world.tilemap[tx][ty] = t;
        address += 3;
    }
    
    public void setSolid(){
        int tx = getIntCommand(address);
        int ty = getIntCommand(address+1);
        int t = getIntCommand(address+2);
        world.tilesolid[tx][ty] = t;
        address += 3;
    }
    
    public void setTint(){
    }
    
    public void spark(){
        world.addObject(new sparks(getIntCommand(address)),commands.get(address+1),commands.get(address+2));
        address += 3;
    }
        
    public void sleep(){
        asleep = true;
        if(debug)System.out.println();
        if(debug)System.out.println("Excecution halted at address: " + address);
        if(debug)System.out.println();
    }
       
    public void pause(){
        if(timer==getIntCommand(address)){
            timer = 0;
            address++;
        }else{
            timer++;
            address--;
        }
    }
        
    public void terminate(){
        asleep = true;
        if(debug)System.out.println("Script: " + this + " terminated");
        getWorld().removeObject(this);
    }
    
    public void loadScript(){
        world.loadScript(renderedStrings.get(getIntCommand(address)));
        address++;
    }
  
    public void startScript(){
        world.startScript(renderedStrings.get(getIntCommand(address)));
        address++;
    }
  
    public void playSound(){
        Greenfoot.playSound(renderedStrings.get(getIntCommand(address)));
        address++;
    }
    
    public void turnOnFlashlight(){
        world.turnOnFlashlight();
    }
    
    public void turnOffFlashlight(){
        world.turnOffFlashlight();
    }
    
    public void flickerFlashlight(){
        world.flickerFlashlight();
    } 
    
    public void turnOnAss(){
        world.turnOnASS();
    }
    
    public void turnOffAss(){
        world.turnOffASS();
    }
    
    public void lockFlashlight(){
        world.lockFlashlight();
    }
    
    public void giveFlashlight(){
        world.giveFlashlight();
    }
    
    public void say(){
        world.say(renderedStrings.get(getIntCommand(address)),renderedStrings.get(getIntCommand(address+1)));
        address += 2;
    }
    
    public void transition(){
        world.transitionToArea(getIntCommand(address),true);
        address += 1;
    }
    
    public void setDarkness(){
        world.setDarkness(getIntCommand(address),getIntCommand(address+1),getIntCommand(address+2));
        address += 3;
    }
    
    public void setLightShadow(){
        world.setLightShadow(getIntCommand(address),getIntCommand(address+1));
        address += 2;
    }
    
    public void turnOffShadows(){
        world.turnOffShadows();
    }
    
    public void turnOnShadows(){
        world.turnOnShadows();
    }
    
    public void setShake(){
        world.setShake(getIntCommand(address));
        address++;
    }
    
    public void wakeTrigger(){
        ((Trigger)declaredObjects.get(getIntCommand(address))).wake();
        address++;
    }
    
    public void endTrigger(){
        ((Trigger)declaredObjects.get(getIntCommand(address))).end();
        address++;
    }
    
    public void giveWeapon(){
        world.giveWeapon(getIntCommand(address));
        address++;
    }
    
    public void playerForce(){
        world.character.force(commands.get(address),commands.get(address+1));
        address += 2;
    }
    
    public int getIntCommand(int index){
        double commandD = commands.get(index);
        return (int)commandD;
    }

}
