import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Rectangle;
import java.awt.Polygon;
import java.awt.Color;
import java.awt.image.ColorConvertOp;
import java.util.List;
import java.awt.Rectangle;


public class Light  extends Actor
{

    GreenfootImage image = new GreenfootImage(400,400);         //displayed image
    BufferedImage buffer = image.getAwtImage();
    Graphics2D graphics = buffer.createGraphics();
    GreenfootImage backImage = new GreenfootImage(400,400);     //intermediate buffer used for blending
    BufferedImage backBuffer = backImage.getAwtImage();
    Graphics2D backGraphics = backBuffer.createGraphics();
    GreenfootImage shadowImage = new GreenfootImage(400,400);   //intermediate buffer used for object shadows
    BufferedImage shadowBuffer = shadowImage.getAwtImage();
    Graphics2D shadowGraphics = shadowBuffer.createGraphics();
    
    GreenfootImage flareLight = new GreenfootImage("Gradient.png");
    BufferedImage flareBuffer = flareLight.getAwtImage();
    GreenfootImage fog = new GreenfootImage("Gradient.png");
    BufferedImage fogBuffer;
    
    AlphaComposite comp_clear = AlphaComposite.getInstance(AlphaComposite.XOR,(float).55);
    AlphaComposite comp_grad = AlphaComposite.getInstance(AlphaComposite.XOR);
    
    AlphaComposite comp_rad2 = AlphaComposite.getInstance(AlphaComposite.DST_OUT,(float).55);
    Composite comp_normal = graphics.getComposite();
    
    final double beamWidth = .45;
    final int fogRadius = 60;
    int[] lx = new int[3];
    int[] ly = new int[3];
    
    
    double[] xs = new double[5];
    double[] ys = new double[5];
    int[] xp = new int[4];
    int[] yp = new int[4];
    
    world world;
    
    
    double lightPower = 60;
    double lightAngle = 0;
    int lightAlpha = 0;
    int redAlpha = 0;
    boolean heldByCharacter = false;
    boolean pressed;
    boolean lightLocked = false;
    boolean lightOn = false;
    boolean lightFlicker = false;
    
    
    public Light(){
        setImage(image);
        fog.scale(fogRadius*2,fogRadius*2);
        fogBuffer = fog.getAwtImage();
        flareLight.scale(Flare.radius*2,Flare.radius*2);
        flareBuffer = flareLight.getAwtImage();
        graphics.setPaint(Color.black);
    }
    
    protected void addedToWorld(World w){
        world = (world)w;
    }
    
    public void act() 
    {
        image.setColor(Color.black);
        image.clear();
        if(Greenfoot.isKeyDown("f") && heldByCharacter  && !lightLocked){
            if(!pressed){
                lightOn = !lightOn;
                if(lightOn && lightPower<40){
                    lightPower -= 5;
                }
                Greenfoot.playSound("Mechanical/switch.mp3");
            }
            pressed = true;
        }else{
            pressed = false;
        }
        
        
        List lightObjects = getWorld().getObjects(StaticLight.class);
        graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,1f));     
        
        for(int i=0;i<lightObjects.size();i++){
            StaticLight light = (StaticLight)(lightObjects.get(i));
            if(light.cornerX()+light.width>0 && light.cornerY()+light.height>0 && light.cornerX()<400 && light.cornerY()<400){
                float brightness = light.getBrightness();
                backImage.clear();
                backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,brightness));
                backGraphics.drawImage(light.getMask(),null,light.cornerX(),light.cornerY());
                
                if(light.animatedShadow != null){
                    backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OUT,light.animatedShadow.brightness));
                    backGraphics.fillPolygon(light.animatedShadow.convertedCoordinates[0],
                                             light.animatedShadow.convertedCoordinates[1],
                                             light.animatedShadow.convertedCoordinates[0].length);
                }
                /*
                 * 
                if(brightness!=1){
                    graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.XOR,brightness));
                    graphics.drawImage(light.getMask(),null,light.cornerX(),light.cornerY()); 
                    //graphics.setComposite(comp_grad);
                }else{
                    graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OUT));
                    graphics.drawImage(light.getMask(),null,light.cornerX(),light.cornerY()); 
                    //graphics.setComposite(comp_grad);
                }  
                if(((world)getWorld()).shadowQuality == 0    ||   !light.castsShadows()){
                    continue;
                }
                */
                List<ActionActor> shadowObjects = light.getObjects();
                
                for(int j=0;j<shadowObjects.size();j++){
                    ActionActor current = shadowObjects.get(j);
                    if(((world)getWorld()).shadowQuality < 2){
                        continue;
                    }
                    if(((world)getWorld()).shadowQuality < 3 && current.isShadow()==2){
                        continue;
                    }
                    if(current.isShadow()!=0 && (light.insideRect(current.getX()+5,current.getY()-5) || 
                                                 light.insideRect(current.getX()+5,current.getY()+5) ||
                                                 light.insideRect(current.getX()-5,current.getY()-5) ||
                                                 light.insideRect(current.getX()-5,current.getY()+5))){       
                        double ang = current.getRotation()/180.0*Math.PI;
                        double cos = Math.cos(ang);
                        double sin = Math.sin(ang);
                        for(int k=0;k<current.getShadowX().length;k++){
                            xs[k] = current.getShadowX()[k]*cos - current.getShadowY()[k]*sin + current.getX();
                            ys[k] = current.getShadowY()[k]*cos + current.getShadowX()[k]*sin + current.getY();
                        }
                        float fade = (float)(world.darkness[1]/255.0);
                        float dist = (float)Math.sqrt((Math.pow(light.screenX()-current.getX(),2)+Math.pow(light.screenY()-current.getY(),2)));
                        if(dist<40){
                            fade = (float)(world.darkness[1]/255.0)*(dist-20)/40;
                            if(fade<0){
                                fade = 0;   //optimize
                            }
                        }
                    
                        dist -= 42;
                        if(dist<1){
                            dist = 1;
                        }
                        double s = light.radius - dist;
                        double c = s/dist;
                        //double sq = ((world)getWorld()).shadowQuality;
                        //fade /= (sq+1);
                    
                        backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OUT,(float)fade));
                        for(int k=0;k<current.getShadowX().length-1;k++){
                            double lightR = 0;
                            double l = 0;
                            //for(double l=0;l<Math.PI*2;l += Math.PI/(sq+1)){
                                xp[0] = (int)xs[k];
                                xp[1] = (int)xs[k+1];
                                yp[0] = (int)ys[k];
                                yp[1] = (int)ys[k+1];
                                xp[3] = (int)(xs[k] + c*(xs[k]-light.screenX()-lightR*Math.cos(l)));
                                xp[2] = (int)(xs[k+1] + c*(xs[k+1]-light.screenX()-lightR*Math.cos(l)));
                                yp[3] = (int)(ys[k] + c*(ys[k]-light.screenY()-lightR*Math.sin(l)));
                                yp[2] = (int)(ys[k+1] + c*(ys[k+1]-light.screenY()-lightR*Math.sin(l)));
                                Polygon poly = new Polygon(xp,yp,4);
                                backGraphics.fill(poly);
                            //}
                        }
                        
                        
                    }
                }
                graphics.drawImage(backBuffer,null,0,0);
            }
        }
        
        //List shadowObjects = world.getObjects(AnimatedShadow.class);
        
        /*
        for(int i=0;i<shadowObjects.size();i++){
            AnimatedShadow shadow = (AnimatedShadow)shadowObjects.get(i);
            graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OVER,shadow.brightness));  
            graphics.fillPolygon(shadow.convertedCoordinates[0],shadow.convertedCoordinates[1],shadow.convertedCoordinates[0].length);     
        }
        
        for(int i=0;i<lightObjects.size();i++){
            StaticLight light = (StaticLight)(lightObjects.get(i));
            if(light.cornerX()+light.width>0 && light.cornerY()+light.height>0 && light.cornerX()<400 && light.cornerY()<400 && light.isFlare){
                float brightness = light.getBrightness();
                graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.XOR,brightness));
                graphics.drawImage(light.getMask(),null,light.cornerX(),light.cornerY()); 
            }
        }
        */
        
   
        
        
        
        if(heldByCharacter){
            if(lightOn){
                lightPower -= .01;
                if(Greenfoot.isKeyDown("g")){
                    lightPower = 2;
                }
                if(lightPower<0){
                    lightPower = 0;
                }
                lightAlpha = 150;
            }else{
                if(lightPower<60){
                    lightPower += .4;
                }else if(lightAlpha>0){
                    lightAlpha -= 1;
                }   
            }
        }
    }  
    
    //public void turnOff(){
    //    lightOn = false;
    //}
    
    //public void turnOn(){
    //    lightOn = true;
    //}
    
    public void setBeam(int x,int y,double r){
        lx[0] = x;
        ly[0] = y;
        r = r/180.0*Math.PI;
        lightAngle = r;
    }
    
    public void setBeamOffset(double f){
        lx[1] = lx[0] + (int)(350*Math.cos(lightAngle+beamWidth+f));
        ly[1] = ly[0] + (int)(350*Math.sin(lightAngle+beamWidth+f));
        lx[2] = lx[0] + (int)(350*Math.cos(lightAngle-beamWidth-f));
        ly[2] = ly[0] + (int)(350*Math.sin(lightAngle-beamWidth-f));
    } 
    
    public void setDarkness(int a){
    }
    
    public boolean hasLight(){
        return lightOn && lightPower>0;
    }
    
    public void setRedAlpha(){
        redAlpha = 100;
    }
    
    public void drawHeldLight(){ 
        backImage.clear();
        if(!world.castShadows){
            return;
        }
        
        if(world.character.holdingFlare()){
            Flare flare = (Flare) world.character.currentFlare;
            //graphics.setComposite(comp_rad2);
            backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,(float)(flare.getBrightness()*.5)));
            backGraphics.drawImage(flareBuffer,null,flare.getX()-Flare.radius,flare.getY()-Flare.radius);
        }
        if(world.character.muzzleFlash!=0){
            backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,world.character.muzzleFlash));
            backGraphics.drawImage(flareBuffer,null,200-Flare.radius,200-Flare.radius);     
        }
        
       
        if(lightOn){
            double flickerMod = world.darkness[4]/255;
            if(lightFlicker || lightPower<1){
                flickerMod *= Math.pow(Math.random(),12);
            }
            backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,(float)(.55*flickerMod)));
            backGraphics.drawImage(fogBuffer,null,lx[0]-fogRadius,ly[0]-fogRadius);
            double sq = world.shadowQuality;
               
            backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,(float)(flickerMod*(Math.random()*.005+.16/(sq+2)))));
                
            for(double f=-.1;f<.1;f+=.1/(sq+2)){
                setBeamOffset(f);
                Polygon poly = new Polygon(lx,ly,3);
                backGraphics.fill(poly);
            } 
        }
        
        int lightCenterX = world.lightx - (int)world.tilex;
        int lightCenterY = world.lighty - (int)world.tiley;
        if(world.character.holdingFlare()){
            lightCenterX = world.character.currentFlare.getX();
            lightCenterY = world.character.currentFlare.getY();
        }
        
        List<ActionActor> shadowObjects = world.getObjects(ActionActor.class);
        backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OUT,(float)1));
        world.shadowBuffer1.setColor(Color.black);
        shadowImage.clear();
        shadowImage.setColor(Color.black);
        
        
        for(int i=0;i<shadowObjects.size();i++){
            ActionActor current = shadowObjects.get(i);
            //if(current.isShadow() == 2){//  &&  world.shadowQuality==0){
                //continue;
            //}
            if(current.isShadow()!=0  &&  !(current instanceof Bill)){
                if(current.getX()>-40 || current.getX()<440 || current.getY()>-40 || current.getY()<440){
                    double ang = current.getRotation()/180.0*Math.PI;
                    double cos = Math.cos(ang);
                    double sin = Math.sin(ang);
                    for(int k=0;k<current.getShadowX().length;k++){
                        xs[k] = current.getShadowX()[k]*cos - current.getShadowY()[k]*sin + current.getX() + .5;
                        ys[k] = current.getShadowY()[k]*cos + current.getShadowX()[k]*sin + current.getY() + .5;
                    }
                    for(int k=0;k<current.getShadowX().length-1;k++){
                        xp[0] = (int)xs[k];
                        xp[1] = (int)xs[k+1];
                        yp[0] = (int)ys[k];
                        yp[1] = (int)ys[k+1];
                    
                    
                    
                        xp[3] = (int)xs[k] + 50*((int)xs[k]-lightCenterX);
                        xp[2] = (int)xs[k+1] + 50*((int)xs[k+1]-lightCenterX);
                        yp[3] = (int)ys[k] + 50*((int)ys[k]-lightCenterY);
                        yp[2] = (int)ys[k+1] + 50*((int)ys[k+1]-lightCenterY);
                        //Polygon poly = new Polygon(xp,yp,4);
                        if(current.isShadow()==1){
                            world.shadowBuffer1.fillPolygon(xp,yp,4);
                        }else if(world.shadowQuality!=0){       
                            shadowImage.fillPolygon(xp,yp,4);
                        }
                        
                    }
                }
            }
        }
        backGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OUT,(float)(world.darkness[1]/255.0)));
        backGraphics.drawImage(shadowBuffer,null,0,0);
        graphics.drawImage(backBuffer,null,0,0);
    }
    
    public void finalize(){
        graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.XOR,(float)1));
        graphics.fill(new Rectangle(400,400));
    }
}
