import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;


public class MiniSpider extends Character
{
    static GreenfootImage image1 = new GreenfootImage("Monsters/MiniSpider/Spider1.png");
    static GreenfootImage image2 = new GreenfootImage("Monsters/MiniSpider/Spider2.png");
    double counter = 0;
    int state = 1;    //0 = wandering
                      //1 = seeking player
                      //2 = laying web
                      //3 = seeking web
    final int STATE_WANDERING   = 0;
    final int STATE_SEEK_PLAYER = 1;
    final int STATE_LAYING_WEB  = 2;
    final int STATE_SEEK_WEB    = 3;
    boolean currentImage = false;
    double animationFrames = 0;
    final static int maxFrame = 10;
    
    Web currentWeb;
    Web secondWeb;
    double webPercent = 0;
    double webDestinationX,webDestinationY;
    int wanderTime = 140;
    
    boolean dead = false;
    int deadTimer = 300;
    
    public MiniSpider(boolean aggressive){
        super();
        if(aggressive){
            state = STATE_SEEK_PLAYER;
        }else{
            state = STATE_WANDERING;
        }
        setImage(image1);
        isSolid = false;
    }  
    
    protected void addedToWorld(World w){
        super.addedToWorld(w);
        width = 1;
        height = 1;
    }
    
    public void act(){
        move(0,0);
        
        if(dead){
            deadTimer--;
            if(deadTimer<255){
                getImage().setTransparency(deadTimer);
            }
            if(deadTimer==0){
                world.removeObject(this);
            }
            return;
        }  
        
        
        if(world.distanceToPlayer(this)<7 && Math.abs(world.character.velx)+Math.abs(world.character.vely) > 1){
            kill();
            if(currentWeb!=null){
                currentWeb.webType = Web.BROKEN;
            }
            return;
        }    
        
        switch(state){
            case 0: wander();break;
            case 1: seekPlayer();break;
            case 2: layWeb();break;
            case 3: seekWeb();break;
        }
        if(world.tileSolid(myX+velx,myY+vely) && state == STATE_WANDERING){
            rotate(.2);
        }
        double animVel = Math.pow(velx,2)+Math.pow(vely,2);
        animationFrames += animVel;
        if(animationFrames>maxFrame){
            if(currentImage){
                setImage(image1);
            }else{
                setImage(image2);
            }
            animationFrames -= maxFrame;
        }
        if(animVel < .01){
            setImage(image1);
        }
        
        super.act();
    }
    
    public void wander(){
        counter += 1;
        if(counter > wanderTime){
            velx = Math.cos(angle);
            vely = Math.sin(angle);
            rotate((Math.random()-.5)*.4);
        }
        if(counter > wanderTime*2){
            counter = 0;
            velx = 0;
            vely = 0;
        }
    }
    
    public void seekPlayer(){
        seek(world.tilex+200,world.tiley+200,1);
        if(world.distanceToPlayer(this) < 15){
            tetherPlayer();
        }
    }
    
    public void seekWeb(){
        double dx = currentWeb.myX*webPercent + world.character.myX*(1-webPercent);
        double dy = currentWeb.myY*webPercent + world.character.myY*(1-webPercent);
        seek(dx,dy,1);
        if(distTo(dx,dy) < 8){
            tetherWeb();
        }
    }
    
    public void layWeb(){
        seek(webDestinationX,webDestinationY,1);
        currentWeb.setLocation(myX,myY);
        if(distTo(webDestinationX,webDestinationY) < 8){
            List<Web> webs = world.getObjects(Web.class);
            int baseWebs = 0;
            for(Web web : webs){
                if(web.webType == Web.BASE){
                    baseWebs++;
                }
            }
                    
            state = STATE_SEEK_PLAYER;
            if(baseWebs>4){// &&  1/webs.size() < Math.random()-.5){
                state = STATE_SEEK_WEB;
                currentWeb = (Web)webs.get(Greenfoot.getRandomNumber(webs.size()));
                while(!(currentWeb.getWebType()==Web.BASE)){
                    currentWeb = (Web)webs.get(Greenfoot.getRandomNumber(webs.size()));
                    if(Greenfoot.isKeyDown("r")){
                        return;
                    }
                }
                secondWeb = (Web)webs.get(Greenfoot.getRandomNumber(webs.size()));
                while(!(secondWeb.getWebType()==Web.BASE) || secondWeb==currentWeb){
                    secondWeb = (Web)webs.get(Greenfoot.getRandomNumber(webs.size()));
                    if(Greenfoot.isKeyDown("r")){
                        return;
                    }
                }
                webPercent = Math.random();
            }
            
        }
    }
    
    public void tetherPlayer(){
        currentWeb = new Web();
        currentWeb.makeBase();
        world.addObject(currentWeb,myX,myY);
        state = STATE_LAYING_WEB;
        double ang = Math.random()*Math.PI*2;
        double[] destWeb = world.castRay(Math.cos(ang),Math.sin(ang),myX,myY,1);
        webDestinationX = destWeb[0];
        webDestinationY = destWeb[1];
    }
    
    public void tetherWeb(){
        currentWeb = new Web(currentWeb,secondWeb);
        world.addObject(currentWeb,myX,myY);
        state = STATE_SEEK_PLAYER;
        //double ang = Math.random()*Math.PI*2;
        //double[] destWeb = world.castRay(Math.cos(ang),Math.sin(ang),myX,myY,1);
        //webDestinationX = destWeb[0];
        //webDestinationY = destWeb[1];
    }
    
    public int radius(){
        return 0;
    }
    
    public void seek(double dx,double dy,double s){
        setRotation(Math.atan2(dy-myY,dx-myX) + (Math.random()-.5)*2);
        velx = Math.cos(angle)*s;
        vely = Math.sin(angle)*s;
    }
    
    public void kill(){
        Greenfoot.playSound("Monsters/Spider/MiniSpiderSquish.mp3");
        setImage(new GreenfootImage("Monsters/MiniSpider/Squish.png"));
        clearImage();
        dead = true;
    }
}
