import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;


public class SolidActor extends Bone
{   
    boolean hasPhysics = true;
    boolean isPushable = true;
    boolean isSolid = true;
    boolean isStuck = false;
    int width,height;

    public SolidActor(){
        super();
    }
    
    public SolidActor(double offX,double offY){           //create master bone
        super(offX,offY);
    }
    
    public SolidActor(double offX,double offY,String image){           //create master bone
        super(offX,offY,image);
    }
    
    protected void addedToWorld(World w){
        super.addedToWorld(w);
        width = getImage().getWidth();
        height = getImage().getHeight();
        if(getCollidingObjects().size()!=0){
            isStuck = true;
            isSolid = false;
        }
    }

    public void act(){
        if(hasPhysics){
            actPhysics();
        }
        if(isStuck){
            if(getCollidingObjects().size()==0){
                isSolid = true;
                isStuck = false;
            }
        }
        super.act();
    }
    
    public void actPhysics(){
        List<SolidActor> actors;

        move(velx,0);
        actors = getCollidingObjects();
        if(actors.size() != 0){
            move(-velx,0);
            
            //if(actors.size()!=0){
            //    System.out.println("OH NOES!   " + this);
            //}
 
            for(SolidActor s : actors){
                collideWithX(s);
            }
        }else if(isIntersectingWall()){
            move(-velx,0);
            velx = -velx*CoR;
        }
        
        move(0,vely);
        actors = getCollidingObjects();
        if(actors.size() != 0){
            move(0,-vely);
            for(SolidActor s : actors){
                collideWithY(s);
            }
        }else if(isIntersectingWall()){
            move(0,-vely);
            vely = -vely*CoR;
        }
        
        
        velx *= .9;
        vely *= .9;
    }
    
    public boolean isIntersectingWall(){
        return world.tileSolid(myX+getImage().getWidth()/2,  myY+getImage().getHeight()/2) || 
               world.tileSolid(myX+getImage().getWidth()/2,  myY-getImage().getHeight()/2) || 
               world.tileSolid(myX-getImage().getWidth()/2,  myY+getImage().getHeight()/2) || 
               world.tileSolid(myX-getImage().getWidth()/2,  myY-getImage().getHeight()/2);
    }

    public boolean isSolid(double x,double y){
        if(!isSolid){
            return false;
        }
        boolean ret = x < myX+getImage().getWidth()/2 &&
                      x > myX-getImage().getWidth()/2 &&
                      y < myY+getImage().getWidth()/2 &&
                      y > myY-getImage().getWidth()/2;
        return ret;
    }
    
    public List<SolidActor> getCollidingObjects(){
        List<SolidActor> actors = world.getObjects(SolidActor.class);
        if(!isSolid){
            return new ArrayList<SolidActor>();
        }
        actors.remove(this);
        for(int i=0;i<actors.size();i++){
            SolidActor actor = actors.get(i);
            if(!isCollidingWith(actor)){
                actors.remove(actor);
                i--;
            }
        }
        return actors;
    }
    
    public boolean isCollidingWith(SolidActor actor){
        if(!actor.isSolid){
            return false;
        }
        return actor.myX + actor.width/2 > myX - width/2  &&
               actor.myX - actor.width/2 < myX + width/2  &&
               actor.myY + actor.height/2 > myY - height/2  &&
               actor.myY - actor.height/2 < myY + height/2;
    }
    
    public void collideWithX(SolidActor actor){
        if(!actor.hasPhysics || !actor.isPushable){
            return;
        }
        double Cr = CoR * actor.CoR;
        
        double myNewVx = (Cr*actor.mass*(actor.velx - velx)  +  mass*velx + actor.mass*actor.velx)  /  (mass + actor.mass);
        //double myNewVy = (Cr*actor.mass*(actor.vely - vely)  +  mass*vely + actor.mass*actor.vely)  /  (mass + actor.mass);
        
        double theirNewVx = (Cr*mass*(velx - actor.velx)  +  mass*velx + actor.mass*actor.velx)  /  (mass + actor.mass);
        //double theirNewVy = (Cr*mass*(vely - actor.vely)  +  mass*vely + actor.mass*actor.vely)  /  (mass + actor.mass);
        
        velx = myNewVx;
        //vely = myNewVy;
        actor.velx = theirNewVx;
        //actor.vely = theirNewVy;
    }
    
    public void collideWithY(SolidActor actor){
        if(!actor.hasPhysics || !actor.isPushable){
            return;
        }
        double Cr = CoR * actor.CoR;
        
        //double myNewVx = (Cr*actor.mass*(actor.velx - velx)  +  mass*velx + actor.mass*actor.velx)  /  (mass + actor.mass);
        double myNewVy = (Cr*actor.mass*(actor.vely - vely)  +  mass*vely + actor.mass*actor.vely)  /  (mass + actor.mass);
        
        //double theirNewVx = (Cr*mass*(velx - actor.velx)  +  mass*velx + actor.mass*actor.velx)  /  (mass + actor.mass);
        double theirNewVy = (Cr*mass*(vely - actor.vely)  +  mass*vely + actor.mass*actor.vely)  /  (mass + actor.mass);
        
        //velx = myNewVx;
        vely = myNewVy;
        //actor.velx = theirNewVx;
        actor.vely = theirNewVy;
    }
    
    
    

}
