import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import javax.sound.sampled.*;
import java.io.File;
import java.util.List;


public class Sound  extends ActionActor
{
    GreenfootSound sound;
    GreenfootSound soundBack;
    String soundName;
    int finalVolume = 50;
    double volume = .5;
    boolean distanceBased = false;
    boolean fadeOut = false;
    int maxRadius;

    boolean fading = false;
    double fadeDestination;
    double fadeVel;
    
    boolean smoothFading = false;
    boolean isBack = false;
    double loopTime = 0;
    double startTime = 0;
    
    
    public Sound(String file){
        sound = new GreenfootSound(file);
        soundName = file;
    }
    
    public Sound(double v,String file){
        sound = new GreenfootSound(file);
        soundName = file;
        volume = v;
        finalVolume = (int)(v*100);
    }
    
    public Sound(String file,int r){
        sound = new GreenfootSound(file);
        soundName = file;
        maxRadius = r;
        distanceBased = true;
        finalVolume = 0;
    }
    
    protected void addedToWorld(World world){
        super.addedToWorld(world);
        fadeOut = false;
    }
    
    public void act(){
        
        if(smoothFading){
            //System.out.println(System.currentTimeMillis() + "   :   " + (startTime+loopTime));
            if(System.currentTimeMillis() > startTime+loopTime){
                startTime = System.currentTimeMillis();
                isBack = !isBack;
                if(isBack){
                    soundBack.stop();
                    soundBack.play();
                    if(!fading && !distanceBased){
                        soundBack.setVolume((int)(volume*100));
                    }
                }else{
                    sound.stop();
                    sound.play();
                    if(!fading && !distanceBased){
                        sound.setVolume((int)(volume*100));
                    }
                }
            }
        }
        if(fading){
            if(Math.abs(volume-fadeDestination)<fadeVel){
                volume = fadeDestination;
                fading = false;
            }
            if(volume < fadeDestination){
                volume += fadeVel;
            }
            if(volume > fadeDestination){
                volume -= fadeVel;
            }
        }
        if(fading || distanceBased){
            if(distanceBased){
                double dist = world.distanceToPlayer(this);
                double mod;
                if(dist>maxRadius){
                    mod = 0;
                }else{
                    mod = 1 - (dist/maxRadius);
                }
                if(finalVolume < (int)(volume*mod*100)){
                    finalVolume++;
                }
                if(finalVolume > (int)(volume*mod*100)){
                    finalVolume--;
                }
            }else{
                if(finalVolume < (int)(volume*100)){
                    finalVolume++;
                }
                if(finalVolume > (int)(volume*100)){
                    finalVolume--;
                }
            }
            //System.out.println(finalVolume + "       :"+volume+":"+fading);
            
            sound.setVolume(finalVolume);
            if(soundBack != null){
                soundBack.setVolume(finalVolume);
            }
            if(fadeOut && finalVolume==0){
                world.removeObject(this);
                sound.stop();
                if(soundBack != null){
                    soundBack.stop();
                }
                return;
            }
        }
        move(0,0);
        super.act();
    }
    
    
    public boolean action(int a){
        actionNumber = a;
        switch(a){
            case 1:sound.play();return true;
            case 2:sound.pause();return true;
            case 3:stop();return true;
            case 4:sound.playLoop();return true;
        }
        return false;
    }
    
    public boolean action(int a,double ... inputs){
        actionNumber = a;
        switch(a){
            case 0:setVolume(inputs[0]);break;
            case 1:fade(inputs[0],inputs[1]);break;
            case 2:smoothLoop(inputs[0]);break;
        }
        return false;
    }

    //destination   frames
    public void fade(double a,double b){
        fading = true;
        fadeDestination = a;
        fadeVel = Math.abs((fadeDestination-volume)/b);
    }
    
    public void fadeOut(double time){
        fade(0,time);
        fadeOut = true;
    }
    
    public void setVolume(double v){
        finalVolume = (int)(v*100);
        volume = v;
        sound.setVolume(finalVolume);
        if(soundBack != null){
            soundBack.setVolume(finalVolume);
        }
    }
    
    public void playLoop(){
        sound.playLoop();
    }
    
    public void play(){
        sound.play();
    }
    
    public void stop(){
        sound.stop();
        if(soundBack != null){
            soundBack.stop();
        }
        smoothFading = false;
    }
    
    public void pause(){
        sound.pause();
    }
    
    public void smoothLoop(double loop){
        //System.out.println("smooth started"+loop);
        loopTime = loop;
        startTime = System.currentTimeMillis();
        soundBack = new GreenfootSound(soundName);
        isBack = false;
        sound.stop();
        soundBack.stop();
        sound.play();
        smoothFading = true;
    }
    
    public void gamePaused(){
        //if(!sound.isPlaying() && state!=3){
        //    state = 0;
        //}
        //sound.pause();
    }
    
    public void gameResumed(){
        //if(state == 1){
        //    sound.play();
        //}
        //if(state == 2){
        //    sound.playLoop();
        //}
    }
    
    public boolean keep(){
        return fadeOut;
    }
  
}
