import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;

public class SpeachBox  extends Actor
{
    GreenfootImage image = new GreenfootImage(400,40);
    GreenfootImage buffer = new GreenfootImage(400,40);
    int timer = 0;
    final int fadeRate = 1;
    final int scrollDistance = 20;
    final int lineStartY = 31;
    final int lineStartX = 2;
    
    ArrayList<Message> messageQueue = new ArrayList<Message>();
    
    public SpeachBox(){
        setImage(image);
    }
    
    public void act() 
    {
        boolean messagesEmpty = true;
        for(Message m : messageQueue){
            m.spendTime();
            messagesEmpty &= m.isTimeUp();
        }
        if(messagesEmpty && !isQueueEmpty()){
            if(image.getTransparency()-fadeRate < 0){
                image.setTransparency(0);
                image.clear();
                clearQueue();
            }else{
                image.setTransparency(image.getTransparency()-fadeRate);
            }   
        }else{
            image.setTransparency(255);
        }
        
        for(Message m : messageQueue){
            if(!m.isDisplayed()){
                buffer.clear();
                buffer.drawImage(image,0,-1);
                image.clear();
                image.drawImage(buffer,0,0);
                timer++;
                if(timer == scrollDistance){
                    timer = 0;
                    displayMessage(m);
                }
                break;
            }
        }

    }   
    
    public void clearQueue(){
    }
    
    public boolean isQueueEmpty(){
        return messageQueue.size()==0;
    }
    
    public void displayMessage(Message m){
        image.setColor(Color.black);
        for(int i=0;i<7;i++){
            for(int j=0;j<5;j++){
                image.drawString(m.getMessage(),i+lineStartX,j+lineStartY);
            }
        }
        image.setColor(m.getColor());
        image.drawString(m.getMessage(),lineStartX+3,lineStartY+2);
        m.display();
    }
    
    public void say(String line,Color color){
        line = line.replace('_',' ');
        messageQueue.add(new Message(line,260,color));
        /*
        buffer.clear();
        buffer.drawImage(image,0,-12);
        image.clear();
        image.drawImage(buffer,0,0);
        image.setColor(Color.black);
        for(int i=2;i<9;i++){
            for(int j=31;j<36;j++){
                image.drawString(line,i,j);
            }
        }
        image.setColor(color);
        image.drawString(line,5,33);
        timer += (int)(Math.sqrt(line.length())*40);
        fade = 255;
        image.setTransparency(255);
        */
    }
}


class Message{
    String myMessage;
    int displayTime;
    Color myColor;
    
    boolean displayed = false;
    int yLocation = 0;

    public Message(String message,int time,Color color){
        myMessage = message;
        displayTime = time;
        myColor = color;
    }
    
    public void spendTime(){
        displayTime--;
    }
    
    public int getTime(){
        return displayTime;
    }
    
    public boolean isTimeUp(){
        return displayTime <= 0;
    }
    
    public boolean isDisplayed(){
        return displayed;
    }
    
    public void display(){
        displayed = true;
    }
    
    public String getMessage(){
        return myMessage;
    }
    
    public Color getColor(){
        return myColor;
    }
}

