import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Rectangle;
import java.awt.Polygon;
import java.awt.Color;
import java.awt.image.ColorConvertOp;
import java.util.List;

public class StaticLight  extends ActionActor
{
    GreenfootImage mask;
    GreenfootImage backBuffer;
    BufferedImage maskBuffer;
    Graphics2D maskGraphics;
    int radius;
    int cornerX,cornerY;
    float brightness = 1;
    AnimatedShadow animatedShadow = null;
    
    boolean flickering = false;
    double flicker = 0;
    int flickerTimer = 0;
    float flickerValue;
    float flickerSplit = (float)(.3);
    float flickerCenter = (float)(.5);
    int updateFrequency = 5;
    int timer = 0;
    boolean castsShadows = true;
    
    double[] xs = new double[5];
    double[] ys = new double[5];
    int[] xp = new int[4];
    int[] yp = new int[4];
    
    int centerX,centerY;
    int width,height;
    
    boolean beam = false;
    double beamAng;
    double beamWidth;
    double beamCaper;
    
    boolean solid = false;
    
    //solid mode
    //  width
    //  height
    //  brightness
    public StaticLight(int w,int h,int cx,int cy,float br){
        mask = new GreenfootImage(w,h);
        maskBuffer = mask.getAwtImage();
        maskGraphics = maskBuffer.createGraphics();
        maskGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR));
        brightness = br;
        mask.setColor(Color.black);
        mask.fill();
        
        centerX = cx;
        centerY = cy;
        width = w;
        height = h;  
        solid = true;
        radius = 9999;
    }
    
    //normal mode
    public StaticLight(int r,float br){
        mask = new GreenfootImage("Gradient.png");
        mask.scale(r*2,r*2);
        maskBuffer = mask.getAwtImage();
        maskGraphics = maskBuffer.createGraphics();
        maskGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR));
        radius = r;
        brightness = br;
        
        centerX = r;
        centerY = r;
        width = r*2;
        height = r*2;    
    }
    
    //bounded mode
    //  radius
    //  brightness
    //  width
    //  height
    //  centerX
    //  centerY
    public StaticLight(int r,float br,int w,int h,int cX,int cY){
        centerX = cX;
        centerY = cY;
        width = w;
        height = h;
        
        mask = new GreenfootImage(width,height);
        GreenfootImage temp = new GreenfootImage("Gradient.png");
        temp.scale(r*2,r*2);
        mask.drawImage(temp,cX-r,cY-r);
        maskBuffer = mask.getAwtImage();
        maskGraphics = maskBuffer.createGraphics();
        maskGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR));
        radius = r;
        brightness = br; 
    }
    
    //beam mode
    //  radius
    //  brightness
    //  angle
    //  beam angle width
    //  beam caper ammount 
    public StaticLight(int r,float br,double ang,double bwidth,double caper){
        ang = ang/180.0*Math.PI;
        bwidth = bwidth/180.0*Math.PI;
        
        mask = new GreenfootImage("Gradient.png");
        mask.scale(r*2,r*2);
        maskBuffer = mask.getAwtImage();
        maskGraphics = maskBuffer.createGraphics();
        
        maskGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR));
        radius = r;
        brightness = br;
        
        centerX = r;
        centerY = r;
        width = r*2;
        height = r*2;  
        
        beamAng = ang;
        beamWidth = bwidth;
        beamCaper = caper;
        beam = true;
    }
    
    //beam bounded mode
    //  radius
    //  brightness
    //  angle
    //  beam angle width
    //  beam caper ammount 
    public StaticLight(int r,float br,double ang,double bwidth,double caper,int leftSide,int rightSide,int topSide,int bottomSide){
        ang = ang/180.0*Math.PI;
        bwidth = bwidth/180.0*Math.PI;
        
        width = leftSide + rightSide;
        height = topSide + bottomSide; 
        mask = new GreenfootImage(width,height);
        GreenfootImage temp = new GreenfootImage("Gradient.png");
        temp.scale(r*2,r*2);
        mask.drawImage(temp,leftSide-r,topSide-r);
        //mask.fill();
        
        maskBuffer = mask.getAwtImage();
        maskGraphics = maskBuffer.createGraphics();
        
        maskGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR));
        radius = r;
        brightness = br;
        
        centerX = leftSide;
        centerY = topSide;
         
        
        beamAng = ang;
        beamWidth = bwidth;
        beamCaper = caper;
        beam = true;
    }
    
    protected void addedToWorld(World w){
        super.addedToWorld(w);
        
        cornerX = (int)myX-width/2;
        cornerY = (int)myY-height/2;
        int tileStartX = cornerX/40*40;
        int tileStartY = cornerY/40*40;
        int tileEndX = cornerX + width + 40;
        int tileEndY = cornerY + height + 40;        
        
        
        double[] dotx = {0,20,0,-20};
        double[] doty = {-20,0,20,0};
        
        int tile;
        int polyCount=0;
        for(int i=tileStartX;i<tileEndX;i+=40){
            for(int j=tileStartY;j<tileEndY;j+=40){
                tile = world.tile(i,j);
                
                int imX = i - cornerX;
                int imY = j - cornerY;
                //System.out.println(i+":"+j + "     " + imX+":"+imY);
                //System.out.println(tile);
                
                if(tile == 0){
                    xs[0] = imX;xs[1] = imX+40;xs[2] = imX+40;xs[3] = imX;xs[4] = imX;
                    ys[0] = imY;ys[1] = imY;ys[2] = imY+40;ys[3] = imY+40;ys[4] = imY;
                    for(int k=0;k<4;k++){
                        xp[0] = (int)xs[k];
                        xp[1] = (int)xs[k+1];
                        yp[0] = (int)ys[k];
                        yp[1] = (int)ys[k+1];
                        if((xp[0]>=0 && xp[0]<width && yp[0]>=0 && yp[0]<height)
                        || (xp[1]>=0 && xp[1]<width && yp[1]>=0 && yp[1]<height)){
                            //double dot1 = (xp[0]-200)*(mx-200) + (yp[0]-200)*(my-200);
                            //double dot2 = (xp[1]-200)*(mx-200) + (yp[1]-200)*(my-200);
                            double dot3 = (dotx[k])*(imX-centerX) + (doty[k])*(imY-centerY);
                            
                            if(dot3<0){
                                xp[3] = (int)xs[k] + 50*((int)xs[k]-centerX);
                                xp[2] = (int)xs[k+1] + 50*((int)xs[k+1]-centerX);
                                yp[3] = (int)ys[k] + 50*((int)ys[k]-centerY);
                                yp[2] = (int)ys[k+1] + 50*((int)ys[k+1]-centerY);
     
                                polyCount++; 
                                Polygon poly = new Polygon(xp,yp,4);
                                maskGraphics.fill(poly);
                                //getBackground().setColor(Color.RED);
                                //getBackground().drawOval(i+5,j+5,3,3);
                            }
                        }
                    }
                    
                }
            }
        } 
        if(beam){
            int[] beamX = {centerX,
                          (int)(centerX+Math.cos(beamAng+beamWidth)*radius*10),
                          (int)(centerX+Math.cos(Math.PI+beamAng-beamWidth)*radius*10),
                          (int)(centerX+Math.cos(Math.PI+beamAng+beamWidth)*radius*10),
                          (int)(centerX+Math.cos(beamAng-beamWidth)*radius*10)};
            int[] beamY = {centerY,
                          (int)(centerY+Math.sin(beamAng+beamWidth)*radius*10),
                          (int)(centerY+Math.sin(Math.PI+beamAng-beamWidth)*radius*10),
                          (int)(centerY+Math.sin(Math.PI+beamAng+beamWidth)*radius*10),
                          (int)(centerY+Math.sin(beamAng-beamWidth)*radius*10)};
            Polygon poly = new Polygon(beamX,beamY,5);
            maskGraphics.fill(poly);
            int[] beamX2 = {centerX,
                           (int)(centerX+Math.cos(beamAng+beamWidth*1.1)*beamCaper),
                           (int)(centerX+Math.cos(beamAng-beamWidth*1.1)*beamCaper)};
            int[] beamY2 = {centerY,
                           (int)(centerY+Math.sin(beamAng+beamWidth*1.1)*beamCaper),
                           (int)(centerY+Math.sin(beamAng-beamWidth*1.1)*beamCaper)};
            poly = new Polygon(beamX2,beamY2,3);
            maskGraphics.fill(poly);
            maskGraphics.draw(poly);
        }
        //mask.fill();
        maskGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_IN,brightness));
        Rectangle rect = new Rectangle(0,0,radius*2,radius*2);
        maskGraphics.fill(rect);
        brightness = 1;
    } 
     
    
    public BufferedImage getMask(){
        return maskBuffer;
    }
    
    public int cornerX(){
        return getX()-width/2;
    }
    
    public int cornerY(){
        return getY()-height/2;
    }
    
    public float getBrightness(){
        if(!flickering){
            return brightness;
        }
        if(timer%updateFrequency == 0){     
            flickerValue = (float)(brightness*flicker*Math.random() + (1-flicker)*brightness);
            if(Math.abs(flickerValue - brightness/2) < flickerSplit){
                if(flickerValue<flickerCenter){
                    flickerValue = (float)brightness/2-flickerSplit;
                }else{
                    flickerValue = (float)brightness/2+flickerSplit;
                }  
            }
            if(flickerValue<0){
                flickerValue = 0;
            }
        }
        if(flickerValue<0)flickerValue = 0;
        if(flickerValue>1)flickerValue = 1;
        return (float)flickerValue;
    }
    
    public boolean action(int a){
        actionNumber = a;
        if(a>9){
            brightness = (float)((a-10)/10.0);
        }
        switch(a){
            case 2: flickerLight();return true;
            case 3: flickerMedium();return true;
            case 4: flickerHeavy();return true;
            case 5: flickerOff();return true;
            case 6: setCanCastShadows(false);return true;
            case 7: setCanCastShadows(false);return true;
        }
        return false;
    }
    
    public boolean action(int a,double ... inputs){
        actionNumber = a;
        switch(a){
            case 1: setFlicker(inputs[0]);return true;
            case 2: setFlickerSplit((float)inputs[0]);return true;
            case 3: setFlickerUpdateFrequency((int)inputs[0]); return true;
            case 4: brightness = (float)inputs[0];return true;
            case 5: flickerCenter = (float)inputs[0];return true;
        }
        return false;
    }
    
    public void setFlickerUpdateFrequency(int i){
        updateFrequency = i;
        flickering = true;
    }
    
    public void setBrightness(double i){
        brightness = (float)i;
    }
    
    public void setFlicker(double i){
        flicker = i;
        flickering = true;
    }
    
    public void setFlickerSplit(float i){
        //System.out.println(i);
        flickerSplit = i;
        flickering = true;
    }
    
    public void flickerLight(){
        flicker = .3;
        flickering = true;
    }
    
    public void flickerMedium(){
        flicker = .6;
        flickering = true;
    }
    
    public void flickerHeavy(){
        flicker = 1;
        flickering = true;
    }
    
    public void flickerOff(){
        flicker = 0;
        flickering = false;
    }
    
    public List getObjects(){
        return getObjectsInRange(radius,ActionActor.class);
    }
    
    public boolean insideRect(int x,int y){
        if(!(x>cornerX() && y>cornerY() && x<cornerX()+width && y<cornerY()+height)){
            return false;
        }
        if(beam){
            return true;
        }
        return mask.getColorAt(x-(int)cornerX(),y-(int)cornerY()).getAlpha()!=0;
    }
    
    public int screenX(){
        return getX() - width/2 + centerX;
    }
    
    public int screenY(){
        return getY() - height/2 + centerY;
    }
    
    public boolean castsShadows(){
        return castsShadows;
    }
    
    public void setCanCastShadows(boolean s){
        castsShadows = s;
    }
        
}
