import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;

public class Web extends ActionActor
{
    static Color alphaColor = new Color(100,100,100,128);
    Bill character;
    final double webStrength = 1000;
    Web web1;
    Web web2;
    
    int webType = 0;
    final static int BASIC = 0;
    final static int BASE = 1;
    final static int CROSS = 2;
    final static int BROKEN = 3;
    double webX1,webY1,webX2,webY2;
    double web1Percent,web2Percent;
    
    int health = 10;
    static boolean cooldown = false;
    int deathTimer = 30;
    
    public Web(){
        webType = BASE;
    }
    
    
    public Web(double x,double y){
        webX1 = x;
        webY1 = y;
        webType = BASIC;  
    }
    
    public void makeBase(){
        webType = BASE;
    }
    
    public Web(Web w1,Web w2){
        webType = CROSS;
        web1 = w1;
        web2 = w2;
        double length1 = Math.sqrt(Math.pow(web1.webX1-web1.webX2,2)+Math.pow(web1.webY1-web1.webY2,2));
        double length2 = Math.sqrt(Math.pow(web2.webX1-web2.webX2,2)+Math.pow(web2.webY1-web2.webY2,2));
        if(length1>length2){
            double temp = length1; length1 = length2; length2 = temp; 
            Web tempW = web1; web1 = web2 ; web2 = tempW;
        }
        double percent = Math.random();
        web1Percent = percent;
        web2Percent = percent*length1/length2;
    }
    
    protected void addedToWorld(World w){
        super.addedToWorld(w);
        character = world.character;
    }
        
    public void act() 
    {
        cooldown = false;
        world.getBackground().setColor(alphaColor);
        if((web1!=null && web1.webType==BROKEN) || (web2!=null && web2.webType==BROKEN)){
            webType = BROKEN;
        }
        if(webType == BASE){
            webX1 = myX;
            webY1 = myY;
            webX2 = character.myX;
            webY2 = character.myY;
            world.character.force((myX - character.myX)/webStrength,(myY - character.myY)/webStrength);
        }
        if(webType == CROSS){
            webX1 = web1.getPercentX(web1Percent);
            webY1 = web1.getPercentY(web1Percent);
            webX2 = web2.getPercentX(web2Percent);
            webY2 = web2.getPercentY(web2Percent);
        }
        if(webType == BASIC){
            webX2 = myX;
            webY2 = myY;
        }
        if(webType == BROKEN){
            webX1 += (webX2 - webX1)/10;
            webX2 += (webX1 - webX2)/10;
            webY1 += (webY2 - webY1)/10;
            webY2 += (webY1 - webY2)/10;
            deathTimer--;
            if(deathTimer<=0){
                world.removeObject(this);
                return;
            }
            
        }
        world.drawLine((int)(webX1),
                       (int)(webY1),
                       (int)(webX2),
                       (int)(webY2),1);
    }   
    
    public int getWebType(){
        return webType;
    }
    
    public double getPercentX(double p){
        return webX1*p + webX2*(1-p);   
    }
    
    public double getPercentY(double p){
        return webY1*p + webY2*(1-p);   
    }
    
    public static void createWeb(world w,double x,double y,int sections){
        List<Web> webs = new ArrayList<Web>();
        for(double i=0;i<Math.PI*2;i+= Math.PI*2/sections){
            double dx = Math.cos(i);
            double dy = Math.sin(i);
            double[] loc;
            loc = w.castRay(dx,dy,x,y,1);
            if(Math.abs(loc[0]-x)<100 && Math.abs(loc[1]-y)<100){
                webs.add(new Web(loc[0],loc[1]));
            }
        }
        for(int i=0;i<webs.size();i++){
            Web web1 = webs.get(i);
            w.addObject(web1,x,y); 
        }
        for(int i=0;i<webs.size();i++){
            Web web1 = webs.get(i);
            Web web2 = webs.get((i+1)%webs.size());
            for(int j=0;j<Greenfoot.getRandomNumber(8)+4;j++){
                Web connection = new Web(web1,web2);
                w.addObject(connection,0.0,0.0);     
            }    
        }
    }
    
    public void setEnd1(double x,double y){
        webX1 = x;
        webY1 = y;
    }
    
    public void setEnd2(double x,double y){
        webX2 = x;
        webY2 = y;
    }
    
    public void hurt(double dx,double dy,double mag){
        if(webType == BROKEN){
            return;
        }  
        if(!cooldown || Math.random()>.7){
            health -= 10;
            if(!cooldown){
                Greenfoot.playSound("Monsters/Spider/WebCut" + Greenfoot.getRandomNumber(3) + ".mp3");
            }
        }
        if(health <= 0){
            if(!cooldown){
                if(Math.random()<.3){
                    Greenfoot.playSound("Monsters/Spider/WebSnap.mp3");
                }
                if(Math.random()<.3){
                    Greenfoot.playSound("Monsters/Spider/WebSlide.mp3");
                }
            }
            webType = BROKEN;
        }  
        cooldown = true;
    }

}
