import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Rectangle;
import java.awt.Polygon;
import java.awt.Color;

import greenfoot.core.WorldHandler;  
import java.awt.event.*;
import javax.swing.*;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import javax.swing.ImageIcon;


public class world extends World
{
    //CHARACTER DATA
    Bill character = new Bill();
    double tilex=0,tiley=0;
    double deltaX,deltaY; 
    
    //QUALITY SETTINGS
    int curveQuality = 2;
    int particleQuality = 0;
    boolean fastTrig = true;
    int musicVolume = 70;
    int shadowQuality = 3;  //3 - Full Shadows
                            //2 - Static Lights don't cast Secondary Objects
                            //1 - Static Lights don't cast Primary Objects
                            //0 - Secondary Objects don't cast shadows
    FPS fpsCounter = new FPS();
    boolean autoQualityAdjust = true;
    
    //LEVEL DATA AND SETTINGS
    static GreenfootImage[] tiles = new GreenfootImage[72];
    static GreenfootImage tileset;
    int areaNumber = 1;
    static GreenfootImage shadowBuffer1 = new GreenfootImage(400,400);
    static GreenfootImage shadowBuffer2 = new GreenfootImage(400,400);
    static GreenfootImage shadowBuffer3 = new GreenfootImage(400,400);
    static ShadowOverlay overlay1; 
    static ShadowOverlay overlay2;
    static ShadowOverlay overlay3;
    Light lightShadow = new Light();
    double[] darkness = {0,0,0,255,255};
    double[] dest = {0,0,0,255,255};
    double[] speed = {0,0,0,0,0};
    int lightx=500,lighty=500;
    boolean castShadows = true;
    String levelShortcuts = "1234567890q";
    boolean hasFlashlight = false;
    int shake = 0;
    SpeachBox sp = new SpeachBox();
    int deathTimer = 0;

    //MOUSE INFO
    MouseInfo mouse;
    int mx,my;
    double mouseAng;
    
    //DEBUGGING VARIABLES
    boolean shadowDebug = false;
      
    //AUTO SCARE SYSTEM
    boolean ASSon = false;
    int ASStimer = 0;
    boolean[] ASSscares = {true,true,true,true,true,true,true};
    
    
    //TILEMAP
    int[][] tilemap;
    int[][] tilesolid;
    ArrayList<Actor> declaredObjects = new ArrayList<Actor>();
    ArrayList<String> objectEquates = new ArrayList<String>();    
    ArrayList<EventScript> loadedScripts = new ArrayList<EventScript>();
    ArrayList<String> scriptEquates = new ArrayList<String>();  
    final static int[] levelMapOffset = {0,1,2,3,4,4,4,5,5,6,6,6};

    
    //CUSTOM CURSOR
    JPanel panel = WorldHandler.getInstance().getWorldCanvas();  
    Cursor customCursor;
    ScrollingListener scroll = new ScrollingListener();
    
    //MAIN MENU
    boolean inMenu = true;
    final int introMax = 1;  //default is 260
    int intro = introMax;
    GreenfootSound menuIntroSound = new GreenfootSound("Ambiance/MenuIntro.mp3");
    Sound menuLoop = new Sound("Ambiance/MenuLoop.mp3");
    GreenfootImage back;// = new GreenfootImage("NH.bmp");
    FastImage NH;// = new FastImage(back);
    FastImage image;
    int menuNumber = 0;     //0 main menu
                            //1 options menu
    MouseImage mouseCursor = new MouseImage();
    int[] levelAreaOffset = {3,4,7,9};
    String[] levelNames = {"EMERGENCE","SURGERY","RECOVERY","MAINTENANCE"};
    boolean firstRun = true;
    menu myMenu;
    
    //HUD
    BasicImage weaponPreview = new BasicImage("Items/Axe.bmp");
    String[] weaponNames = {"Axe","Pistol","Magnum","M16","Shotgun"};
 
    
    public world()
    {    
        super(400, 400, 1,false);  
        //Greenfoot.playSound("Ambiance/WaitingRoom.mp3");
        addObject(lightShadow,200,200);
        addActionActor(character,0,0);
        
        overlay1 = new ShadowOverlay(shadowBuffer1,this);
        overlay2 = new ShadowOverlay(shadowBuffer2,this);
        overlay3 = new ShadowOverlay(shadowBuffer3,this);
        

        setPaintOrder(MouseImage.class,BasicImage.class,TextMessage.class,Pressable.class,force.class,sparks.class,SpeachBox.class,FPS.class,ShadowOverlay.class,AboveShadows.class,Light.class,mist.class,StaticOverlay.class,ActionActor.class,bodyPartAbove.class,Zombie.class,Character.class,Person.class,bodyPartBelow.class,Crate.class,Lever.class,MiniSpider.class,Claw.class,Cable.class,Flare.class,BloodSplatter.class,shrapnel.class,BloodSmears.class,BloodSlide.class,StaticImage.class);
        setActOrder(StaticLight.class,ShadowOverlay.class,Character.class,bodyPartAbove.class,bodyPartBelow.class,Shadow.class,Person.class,Connector.class,Trigger.class,TriggerAND.class,EventScript.class,AnimatedShadow.class,ActionActor.class,Door.class,Light.class,End.class,Grim.class);
        //addObject(overlay2,200,200);
        addObject(overlay3,200,200);
        addObject(overlay1,200,200);
        shadowBuffer3.fill();
        shadowBuffer2.fill();
        shadowBuffer1.setTransparency((int)darkness[0]);
        shadowBuffer2.setTransparency((int)darkness[1]);
        shadowBuffer3.setTransparency((int)darkness[3]);
        lightShadow.getImage().setTransparency((int)darkness[2]);
        addObject(new End(),0,0);
        addObject(sp,200,380);
        getBackground().setColor(Color.black);
        getBackground().fill();
        panel.addMouseWheelListener(scroll);
        resetCursor();
        addObject(fpsCounter,24,385);
        addObject(weaponPreview,50,50);
        weaponPreview.getImage().setTransparency(0);
        //myMenu.setChildWorld(this);
        
    }
    
    public void act(){
 
        if(Greenfoot.mousePressed(null)){
            enableCustomCursor();
        }
        if(Greenfoot.getMouseInfo()!=null){
            mouse = Greenfoot.getMouseInfo();
            mx = mouse.getX();
            my = mouse.getY();
            mouseAng = Math.atan2(my-200,mx-200);
        }
        runGame();
        if(Greenfoot.isKeyDown("m") && Greenfoot.mousePressed(null)){
            MouseInfo mouse = Greenfoot.getMouseInfo();
            System.out.println(mouse.getX()+tilex + " , " + (mouse.getY()+tiley));
            System.out.println(tile(mouse.getX()+tilex,mouse.getY()+tiley));
        }
        if(weaponPreview.getImage().getTransparency()!=0){
            weaponPreview.getImage().setTransparency(weaponPreview.getImage().getTransparency()-2);
        }
    }
    
    
    public void runGame(){
        if(Greenfoot.isKeyDown("escape")){
            loadMenu();
            return;
        }
        if(ASSon){
            ASS();
        }
        character.scrollWeapon(scroll.getScroll());
        tilex = (int)character.myX-200;
        tiley = (int)character.myY-200;
        deltaX = 0;
        deltaY = 0;
        if(shake!=0){
            deltaX = Math.random()*shake*2-shake;
            deltaY = Math.random()*shake*2-shake;
            tilex -= deltaX;
            tiley -= deltaY;
            character.velx += deltaX/10;
            character.vely += deltaX/10;
        }
        deltaX += character.velx;
        deltaY += character.vely;        
        if(Greenfoot.mousePressed(null) && panel.getCursor()!=customCursor){
            panel.setCursor(customCursor);
        }
        String key = Greenfoot.getKey();
        if(key!=null && key.equals("v")){
            shadowDebug = !shadowDebug;
        }
        if(Greenfoot.isKeyDown("l") && key!=null && !key.equals("l")){
            character.removeAllBones();
            character = new Bill();
            addActionActor(character,0,0);
            loadArea(levelShortcuts.indexOf(key)+1);
        }
        if(Greenfoot.isKeyDown("c")){
            //System.out.println(getObjects(null));
        }

        if(hasFlashlight){
            
        }
        if(Greenfoot.isKeyDown("m") && Greenfoot.mousePressed(null)){
            MouseInfo mouse = Greenfoot.getMouseInfo();
            //System.out.println(mouse.getX()+tilex + " , " + (mouse.getY()+tiley));
            //System.out.println(tile(mouse.getX()+tilex,mouse.getY()+tiley));
        }
        if(Greenfoot.isKeyDown("z") && Greenfoot.mousePressed(null)){
            MouseInfo mouse = Greenfoot.getMouseInfo();
            int mmx = (int)(mouse.getX()+tilex);
            int mmy = (int)(mouse.getY()+tiley);
            //Web.createWeb(this,mmx,mmy,12);
            addActionActor(new MiniSpider(true),mmx,mmy);
        }
        
        
        
        render();
        //shadow();
    }  
    
    public void render(){
        getBackground().setColor(Color.black);
        getBackground().fill();
        getBackground().setColor(Color.black);
        for(int i=-((int)tilex%40);i<400;i+=40){
            for(int j=-((int)tiley%40);j<400;j+=40){
                int tile = tile(tilex+i+40-1,tiley+j+40-1);
                if(tile!=0){
                    getBackground().drawImage(tiles[tile],i,j);
                }else{
                    //shadowBuffer1.fillRect(i,j,40,40);
                }
            }
        }  
    }
    
    
    
    public void shadow(){   
        shadowBuffer1.clear();
        shadowBuffer2.clear();
        shadowBuffer1.setColor(Color.black);
        shadowBuffer2.setColor(Color.black);
        int lightCenterX;
        int lightCenterY;
        
        double[] dotx = {0,20,0,-20};
        double[] doty = {-20,0,20,0};
        double[] xs = new double[5];
        double[] ys = new double[5];
        int[] xp = new int[4];
        int[] yp = new int[4];
        
        for(int i=0;i<darkness.length;i++){
            double prev = darkness[i];
            if(Math.abs(darkness[i]-dest[i])>.1){
                if(Math.abs(darkness[i]-dest[i]+speed[i])  >  Math.abs(darkness[i]-dest[i]+speed[i])){
                    darkness[i] -= speed[i];
                }else{
                    darkness[i] += speed[i];
                }
            }
            if(Math.abs(darkness[i]-dest[i]-.1)<speed[i]){
                darkness[i]  = dest[i];
            }
            if(shadowQuality == 0){
                darkness[1] = 0;
            }
            if(Math.abs(prev-darkness[i])>.0001){
                switch(i){
                    case 0: overlay1.setTransparency((int)darkness[0]);break;
                    case 1: overlay2.setTransparency((int)darkness[1]);break;
                    case 3: overlay3.setTransparency((int)darkness[3]);break;
                    case 2: lightShadow.getImage().setTransparency((int)darkness[2]);break;
                }   
            }
        }
        lightCenterX = lightx - (int)tilex;
        lightCenterY = lighty - (int)tiley;
        if(character.holdingFlare()){
            lightCenterX = character.currentFlare.getX();
            lightCenterY = character.currentFlare.getY();
        }
        int centerX = 200+(int)tilex,centerY = 200+(int)tiley;
        
        if(!castShadows){
            for(int i=-((int)tilex%40);i<400;i+=40){
                for(int j=-((int)tiley%40);j<400;j+=40){
                    int tile = tile(tilex+i+40-1,tiley+j+40-1);
                    if(tile==0){
                        shadowBuffer1.fillRect(i,j,40,40);
                    }
                }
            }
            return;
        }
        

        int polyCount = 0;
        
        int tile;
        for(int i=-((int)tilex%40);i<400;i+=40){
            for(int j=-((int)tiley%40);j<400;j+=40){
                tile = tile(tilex+i,tiley+j);

                if(tile == 0){
                    xs[0] = i;xs[1] = i+40;xs[2] = i+40;xs[3] = i;xs[4] = i;
                    ys[0] = j;ys[1] = j;ys[2] = j+40;ys[3] = j+40;ys[4] = j;
                    for(int k=0;k<4;k++){
                        xp[0] = (int)xs[k];
                        xp[1] = (int)xs[k+1];
                        yp[0] = (int)ys[k];
                        yp[1] = (int)ys[k+1];
                        
                        if(((xp[0]>=0 && xp[0]<400 && yp[0]>=0 && yp[0]<400)
                        || (xp[1]>=0 && xp[1]<400 && yp[1]>=0 && yp[1]<400)) &&
                            tile(tilex+i+20+dotx[k]*2,tiley+j+20+doty[k]*2)!=0){              
                            //double dot1 = (xp[0]-200)*(mx-200) + (yp[0]-200)*(my-200);
                            //double dot2 = (xp[1]-200)*(mx-200) + (yp[1]-200)*(my-200);
                            double dot3 = (dotx[k])*(i-lightCenterX) + (doty[k])*(j-lightCenterY);
                            
                            if(dot3<0){
                                xp[3] = (int)xs[k] + 50*((int)xs[k]-lightCenterX);
                                xp[2] = (int)xs[k+1] + 50*((int)xs[k+1]-lightCenterX);
                                yp[3] = (int)ys[k] + 50*((int)ys[k]-lightCenterY);
                                yp[2] = (int)ys[k+1] + 50*((int)ys[k+1]-lightCenterY);
     
                                polyCount++;
                                shadowBuffer1.fillPolygon(xp,yp,4);
                                if(shadowDebug){
                                    shadowBuffer2.setColor(Color.RED);
                                    shadowBuffer2.drawPolygon(xp,yp,4);
                                    shadowBuffer2.setColor(Color.GREEN);
                                    shadowBuffer2.drawLine((int)(i+20+dotx[k]),(int)(j+20+doty[k]),(int)(i+20+dotx[k]*2),(int)(j+20+doty[k]*2));
                                }
                            }
                        }
                    } 
                }
            }
        }  
        lightShadow.getImage().setColor(Color.BLACK);
        //System.out.println(polyCount);
        
        
    }
    
    public void castTileShadow(){
    }
    
    public boolean solidSmooth(double x,double y){
        
        if((x%40<8 || x%40>32) && (y%40<8 || y%40>32)){
            return false;
        }
        return solid(x,y);
    }
    
    public boolean solid(double x,double y){
        if(Greenfoot.isKeyDown("n")){
            return false;
        }
        double tx = x/40;
        double ty = y/40;
        if(tx>tilesolid.length-1 || tx<0 || ty>tilemap[(int)tx].length-1 || ty<0){
            return true;
        }
        boolean solid = false;
        for(Object a : getObjects(SolidActor.class)){
            solid |= ((SolidActor)a).isSolid(x,y); 
        }
        return solid || tilesolid[(int)tx][(int)ty]!=0;
    }
    
    public boolean tileSolid(double x,double y){
        if(Greenfoot.isKeyDown("n")){
            return false;
        }
        x /= 40;
        y /= 40;
        if(x>tilesolid.length-1 || x<0 || y>tilemap[(int)x].length-1 || y<0){
            return true;
        }
        return tilesolid[(int)x][(int)y]!=0;        
    }
    
    public boolean tileSolid(double x,double y,int a){
        x /= 40;
        y /= 40;
        if(x>tilesolid.length-1 || x<0 || y>tilemap[(int)x].length-1 || y<0){
            return true;
        }
        if(tilesolid[(int)x][(int)y] == a){
            return false;
        }else{
            return tilesolid[(int)x][(int)y]!=0;
        }
    }
        
    public boolean hit(double x,double y){
        if(x>tilemap.length-1 || x<0 || y>tilemap[(int)x].length-1 || y<0){
            return true;
        }
        return tilemap[(int)x][(int)y]==1;
    }
    
    public int tile(double x,double y){
        x /= 40;
        y /= 40;
        if(x>tilemap.length-1 || x<0 || y>tilemap[(int)x].length-1 || y<0){
            return 0;
        }
        return tilemap[(int)x][(int)y];
    }
    
    public void turnOffFlashlight(){
        lightShadow.lightOn = false;
        lightShadow.lightLocked = false;
        lightShadow.lightFlicker = false;
    }
    
    public void turnOnFlashlight(){
        lightShadow.lightOn = true;
        lightShadow.lightLocked = false;
        lightShadow.lightFlicker = false;
    }
    
    public void lockFlashlight(){
        lightShadow.lightLocked = true;
    }
    
    public void flickerFlashlight(){
        lightShadow.lightFlicker = !lightShadow.lightFlicker;
        //turnOnFlashlight();
    }
    
    public void loadScript(String file){
        loadedScripts.add(new EventScript(file,this));
        scriptEquates.add(file);
    }
    
    public void startScript(String file){
        addObject(loadedScripts.get(scriptEquates.indexOf(file)),0,0);
    }
    
    public void giveFlashlight(){
        character.giveFlashlight();
        hasFlashlight = true;
        //lightShadow.heldByCharacter = true;
        lightShadow.heldByCharacter = true;
    }
    
    public void giveWeapon(int i){
        if(!character.weapon){
            character.giveWeapon();
        }
        character.switchWeapon(i);
    }
    
    public void stopped(){
        List sounds = getObjects(Sound.class);
        for(int i=0;i<sounds.size();i++){
            ((Sound)sounds.get(i)).gamePaused();
        }
    }
    
    public void started(){
        List sounds = getObjects(Sound.class);
        for(int i=0;i<sounds.size();i++){
            ((Sound)sounds.get(i)).gameResumed();
        }
    }
    
    public void say(String line,String color){
        if(color.equals("red"))sp.say(line,Color.red);
        if(color.equals("blue"))sp.say(line,new Color(128,128,255));
        if(color.equals("green"))sp.say(line,Color.green);
        if(color.equals("yellow"))sp.say(line,Color.yellow);
        if(color.equals("white"))sp.say(line,Color.white);
        if(color.equals("magenta"))sp.say(line,Color.magenta);
    }
    
    public void ASS(){
        ASStimer++;
        if(ASStimer == 18000  || Greenfoot.isKeyDown("p")){
            if(ASStimer==1){
                ASStimer = 0;
                return;
            }
            ASStimer = 0;
            int r = (int)(Math.random()*ASSscares.length);
            while(!ASSscares[r]){
                r = (int)(Math.random()*ASSscares.length);
            }
            EventScript ASSscript = new EventScript("ASS"+r+".txt",this);
            addObject(ASSscript,0,0);
            
            ASSscares[r] = false;
            boolean finished = true;
            for(int i=0;i<ASSscares.length;i++){
                finished = !ASSscares[i] && finished;
            }
            if(finished){
                for(int i=0;i<ASSscares.length;i++){
                    ASSscares[i] = true;
                }
            }
        }
    }      
    
    public void turnOnASS(){
        ASSon = true;
    }
    
    public void turnOffASS(){
        ASSon = false;
    }
    
    public void savePlayerGame(){
        myMenu.farthestLevel = Math.max(myMenu.farthestLevel,areaNumber);
        if(UserInfo.isStorageAvailable()){
            UserInfo myInfo = UserInfo.getMyInfo();
            for(int i=0;i<4;i++){
                myInfo.setInt(i,character.ammo[i] + character.extraAmmo[i]);
            }
            myInfo.setInt(4,character.health);
            myInfo.setInt(5,areaNumber); 
            if(areaNumber > myInfo.getInt(6)){
                myInfo.setInt(6,areaNumber);    
            }
            myInfo.store();
        }
        
    }
    
    public void loadPlayerGame(){ 
        character.removeAllBones();
        character = new Bill();
        addActionActor(character,0,0);
        if(UserInfo.isStorageAvailable()){
            UserInfo myInfo = UserInfo.getMyInfo();
            for(int i=0;i<4;i++){
                character.ammo[i] = Math.min(myInfo.getInt(i),Bill.ammoPerClip[i]);
                character.extraAmmo[i] = Math.max(myInfo.getInt(i)-Bill.ammoPerClip[i],0);
            }
            character.health = myInfo.getInt(4);
            loadArea(myInfo.getInt(5));   
        }else{
            loadArea(areaNumber);     
        }    
    }
    
    public void HUD(){
        if(lightShadow.heldByCharacter){
            int health = getHealth();
            shadowBuffer1.setColor(new Color(128,128,128,lightShadow.lightAlpha));
            shadowBuffer1.fillRect(10,10,(int)lightShadow.lightPower,10);
            shadowBuffer1.fillRect(70,10,1,10);
            if(health*2 >= 0){
                shadowBuffer1.setColor(new Color(128,health*2,health*2,150));
                shadowBuffer1.fillRect(330,10,health,10);
                shadowBuffer1.fillRect(390,10,1,10);
            }
        }
        if(character.health<=0){
            deathTimer++;
        }
        if(deathTimer == 150){
            deathTimer = 0;
            character.removeAllBones();
            character = new Bill();
            addActionActor(character,0,0);
            //loadArea(areaNumber);
            loadPlayerGame();
        }
        if(character.health<=0  && lightShadow.redAlpha!=255){
            lightShadow.redAlpha += 1;
        }
        if(lightShadow.redAlpha>0){
            shadowBuffer1.setColor(new Color(255,0,0,lightShadow.redAlpha));
            shadowBuffer1.fillRect(0,0,400,400);
            if(character.health>0){
                lightShadow.redAlpha -= 1;
            }
        }
        if(character.weaponType>=2){
            shadowBuffer1.drawString(""+character.ammo[character.weaponType-2],50,35);
            shadowBuffer1.drawString(""+character.extraAmmo[character.weaponType-2],340,35);
        }
    }
    
    
    
    
    public void loadArea(int a){
        declaredObjects.clear();
        objectEquates.clear();
        loadedScripts.clear();
        scriptEquates.clear();
        List actors = getObjects(ActionActor.class);
        for(int i=0;i<actors.size();i++){
            ((ActionActor)actors.get(i)).keep = false;
            if(!((ActionActor)actors.get(i)).keep()){
                removeObject(((ActionActor)actors.get(i)));
            }
        }
        actors = getObjects(EventScript.class);
        for(int i=0;i<actors.size();i++){
            removeObject(((EventScript)actors.get(i)));
        }
        EventScript loadScript = new EventScript("Area"+a+"/LoadScript.txt",this);
        addObject(loadScript,0,0);
        loadScript.act();
        removeObject(loadScript);
        actors = getObjects(ActionActor.class);
        for(int i=0;i<actors.size();i++){
            ((ActionActor)actors.get(i)).keep = true;
        }
        transitionToArea(a,false);
        if(a==1){
            character.setBoneLocation(440,740);
        }
        if(a==2){
            character.setBoneLocation(180,740);
        }
        if(a==3){
            character.setBoneLocation(745,460);
        }
        if(a==4){
            character.setBoneLocation(1040,800);
        }
        if(a==5){
            character.setBoneLocation(290,173);
        }
        if(a==6){
            character.setBoneLocation(180,200);
        }
        if(a==7){
            character.setBoneLocation(160,360);
        }
        if(a==8){
            character.setBoneLocation(320,700);
        }
        if(a==9){
            character.setBoneLocation(300,860);
        }
    }
    
    public void transitionToArea(int a,boolean offset){  
        areaNumber = a;
        savePlayerGame();
        overlay3.getImage().fill();
        int offx=0,offy=0;
        int solidInt = 24;
        if(a==1){
            offx = 0;
            offy = 0;
            int[][] a1 = {{0,6,6,6,6,6,0,1,1,1,1,0,0,0,1,1,1,0,0,0,},
                          {0,6,6,6,6,19,0,21,22,22,23,0,0,0,23,21,21,0,0,0,},
                          {0,6,0,0,6,20,0,22,23,1,1,0,0,0,21,22,21,0,0,0,},
                          {0,6,0,0,0,0,0,0,0,0,2,0,0,0,0,0,2,0,0,0,},
                          {0,6,6,6,6,15,0,1,1,1,1,1,1,1,1,1,1,0,0,0,},
                          {0,6,6,6,19,16,0,2,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,0,0,20,16,0,4,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,0,0,0,0,0,1,1,29,1,28,28,1,28,28,24,0,0,0,},
                          {0,6,6,6,16,19,0,1,1,35,1,1,1,1,1,1,26,0,0,0,},
                          {0,6,6,6,16,20,0,1,1,36,1,1,1,37,34,1,26,0,0,0,},
                          {0,6,0,0,18,6,0,1,1,29,1,1,1,1,1,1,3,5,3,5,},
                          {0,6,0,0,0,0,0,1,1,36,1,1,1,1,1,1,3,5,3,5,},
                          {0,6,6,6,15,19,0,1,33,30,1,1,1,1,1,1,26,0,0,0,},
                          {0,6,6,6,16,20,0,1,1,1,1,1,1,1,1,1,26,0,0,0,},
                          {0,6,0,0,17,6,0,39,1,1,1,1,41,1,27,27,24,0,0,0,},
                          {0,6,0,0,0,0,0,0,0,2,2,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,6,6,6,6,6,0,8,8,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,6,6,6,6,6,0,6,6,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,10,10,0,6,6,0,0,11,11,0,11,11,0,0,},
                          {0,6,6,6,16,0,8,8,0,6,6,0,0,11,11,0,11,11,0,0,},
                          {0,15,6,6,18,0,6,6,6,6,6,0,0,12,12,0,12,12,0,0,},
                          {0,16,42,44,46,0,6,6,6,6,6,0,6,6,6,6,6,6,0,0,},
                          {0,16,43,45,47,0,0,0,0,6,6,9,6,6,6,6,6,6,0,0,},
                          {0,16,6,6,6,0,0,0,0,6,6,9,6,6,6,6,6,6,0,0,},
                          {0,17,6,6,6,0,0,0,0,6,6,0,6,6,6,6,6,6,0,0,},
                          {0,6,6,6,6,6,7,9,6,6,6,0,0,13,13,0,13,13,0,0,},
                          {0,6,6,6,6,6,7,9,6,6,6,0,0,11,11,0,11,11,0,0,},
                          {0,0,0,0,0,0,0,0,0,6,6,0,0,11,11,0,11,11,0,0,},
                          {0,0,0,0,0,0,0,0,0,10,10,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,8,8,0,0,0,0,0,0,0,0,0,},
                          {6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,0,},
                          {6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}};
            tilemap = a1;
            solidInt = 24;
        }
        if(a==2){
            offx = -560;
            offy = 200;
            int[][] a2 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,0,0,0,0,0,0,23,23,21,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,0,0,0,0,0,0,23,22,14,14,0,0,0,0,0,0,0,0,0,0,0,6,6,14,14,14,14,0,0,},
                          {0,6,6,0,0,0,0,0,0,21,14,14,21,0,0,0,0,0,0,0,0,0,0,0,6,6,0,22,14,22,0,0,},
                          {0,6,6,0,0,0,0,0,0,22,21,14,14,0,0,0,0,0,11,11,0,11,11,0,6,6,0,14,21,21,0,0,},
                          {0,6,6,0,0,0,0,0,0,21,14,14,14,14,14,14,0,0,11,11,0,11,11,0,6,6,0,22,22,23,0,0,},
                          {0,6,6,0,0,0,0,0,0,22,23,14,14,0,0,14,0,0,12,12,0,12,12,0,10,10,0,0,0,0,0,0,},
                          {0,6,6,0,14,14,14,14,14,14,14,14,14,0,0,14,0,0,6,6,6,6,6,0,8,8,0,19,16,19,6,0,},
                          {0,6,6,0,14,0,0,0,0,0,0,0,0,0,0,14,0,0,6,6,6,6,6,6,6,6,0,20,16,20,6,0,},
                          {0,6,6,0,14,14,6,6,6,3,3,3,16,0,23,14,23,0,6,6,6,6,6,6,6,6,0,1,6,6,6,0,},
                          {0,6,6,0,14,0,0,6,6,6,6,1,18,0,22,14,23,0,6,6,6,6,6,0,6,6,0,19,16,19,6,0,},
                          {0,6,6,0,14,32,30,6,43,43,43,13,13,0,21,14,23,0,13,13,0,13,13,0,6,6,0,20,16,20,6,0,},
                          {0,6,6,0,14,34,29,6,43,43,43,16,6,0,22,22,21,0,11,11,0,11,11,0,6,6,0,6,18,5,6,0,},
                          {0,6,6,0,14,14,28,6,6,6,6,16,2,0,21,21,21,0,11,11,0,11,11,0,6,6,0,0,8,0,0,0,},
                          {0,6,6,0,14,34,26,6,6,6,6,15,1,0,0,0,0,0,0,0,0,0,0,0,6,6,0,6,6,0,0,0,},
                          {0,6,6,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,6,0,6,6,0,0,0,},
                          {0,6,7,9,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,9,6,6,0,4,6,9,6,6,0,},
                          {0,6,7,9,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,9,6,6,0,6,6,9,6,6,0,},
                          {0,6,6,0,0,0,0,0,0,0,0,0,6,6,6,6,6,0,0,0,0,32,31,30,6,6,0,4,6,9,6,6,0,},
                          {0,6,6,0,0,0,0,0,32,31,30,0,6,6,6,6,6,0,14,14,14,14,14,27,6,6,0,6,6,0,0,0,},
                          {0,6,6,0,0,14,14,14,33,14,24,0,6,6,6,6,6,0,14,14,14,14,34,24,6,6,0,6,6,0,0,0,},
                          {0,6,6,0,0,14,14,14,14,34,25,0,6,6,6,6,6,0,14,14,14,14,14,26,6,6,0,6,6,0,0,0,},
                          {0,6,6,0,0,22,14,14,14,14,24,0,6,6,6,6,6,14,14,14,14,14,14,0,6,6,0,6,6,0,0,0,},
                          {0,6,6,0,0,23,21,14,14,14,24,0,6,6,6,6,6,0,14,14,14,14,14,0,10,10,0,0,10,0,0,0,},
                          {0,6,6,0,0,0,0,0,14,14,0,0,0,0,0,0,0,0,0,14,0,0,0,0,14,14,0,3,6,16,6,16,0,},
                          {0,0,0,0,0,0,0,0,14,14,0,0,0,0,0,0,0,0,0,14,0,0,0,0,14,14,0,19,6,19,6,19,0,},
                          {0,0,0,0,0,0,0,0,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,0,20,6,20,6,20,0,},
                          {0,0,0,0,0,0,0,0,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,0,2,6,5,6,5,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}};

            tilemap = a2;
            solidInt = 23;
        }
        if(a==3){
            offx = 360;
            offy = 120;
            int[][] a3 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,7,27,27,27,27,28,27,0,41,46,27,40,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,0,0,0,0,0,27,0,41,27,27,27,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,0,45,46,47,0,27,0,42,27,27,27,27,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,0,43,38,27,0,27,27,27,27,27,27,27,27,27,0,0,0,0,0,0,0,0,0,0,28,28,0,},
                          {0,6,6,0,44,28,27,0,0,0,0,0,0,0,0,0,27,0,0,0,0,0,0,0,0,0,0,28,28,0,},
                          {0,6,6,0,45,27,27,27,27,27,27,27,27,27,27,27,27,0,0,0,6,6,25,25,25,25,27,27,27,0,28,},
                          {0,6,6,0,0,0,0,0,0,0,0,0,27,0,0,0,0,0,0,0,6,6,25,25,25,25,27,27,27,0,28,},
                          {0,6,6,0,0,0,0,28,27,27,27,0,27,41,46,45,47,0,0,0,6,6,0,0,0,0,0,27,27,0,0,},
                          {0,6,6,9,16,0,0,27,0,0,27,0,27,41,45,27,43,0,0,0,6,6,26,26,26,26,0,27,27,0,0,},
                          {0,6,6,9,16,0,0,27,0,0,27,0,27,42,27,27,44,0,0,0,6,6,26,26,26,26,0,27,27,0,0,},
                          {0,6,6,9,18,0,0,27,0,0,27,0,27,27,27,27,27,27,28,0,0,0,0,0,0,0,0,27,27,0,0,},
                          {0,6,6,9,6,0,0,27,0,0,27,0,0,0,0,0,0,0,27,0,0,0,0,0,0,0,0,8,8,0,0,},
                          {0,6,19,9,6,0,0,27,0,0,27,27,28,27,27,28,28,27,28,0,0,12,12,0,12,12,0,6,6,0,0,},
                          {0,6,6,9,21,0,0,27,0,0,0,0,0,0,0,0,0,0,0,0,0,23,23,6,23,23,0,6,6,0,0,},
                          {0,6,6,0,0,0,0,27,0,0,0,0,0,0,0,0,0,0,0,0,0,6,6,20,6,6,7,22,6,0,0,},
                          {0,6,6,0,0,0,0,27,27,6,6,6,6,3,3,6,0,0,0,0,0,6,6,6,20,6,7,6,6,0,0,},
                          {0,6,6,0,0,0,0,27,0,0,6,6,6,6,6,1,0,0,0,0,0,23,23,6,21,23,0,6,6,0,0,},
                          {0,6,6,0,0,0,0,27,36,35,6,5,6,5,6,2,0,0,0,0,0,13,13,0,13,13,0,6,6,0,0,},
                          {0,6,6,0,0,0,0,27,27,33,6,5,6,5,6,21,0,0,0,0,0,0,0,0,0,0,0,6,6,0,0,},
                          {0,6,6,0,0,0,0,27,39,30,6,6,20,6,6,6,0,0,0,0,0,0,0,0,0,0,0,6,6,0,0,},
                          {0,6,6,0,0,0,0,27,27,32,22,6,6,23,6,20,0,0,0,0,0,0,0,0,0,0,0,6,6,0,0,},
                          {0,6,6,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,2,0,},
                          {0,6,6,7,9,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,22,6,7,9,6,6,0,0,},
                          {0,6,6,7,9,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,9,6,6,0,0,},
                          {0,6,6,0,0,0,0,0,0,0,0,0,0,0,0,6,6,6,6,6,0,0,0,0,37,36,35,6,6,0,0,},
                          {0,6,6,0,27,27,28,28,0,27,28,27,27,27,0,6,6,19,6,6,0,40,40,27,38,27,34,6,6,0,0,},
                          {0,6,6,0,27,38,28,43,0,27,38,27,27,27,0,6,23,6,6,6,0,27,27,27,24,27,32,6,6,0,},
                          {0,6,6,0,28,27,28,44,0,28,27,28,28,27,0,6,6,6,6,6,0,27,27,27,24,39,31,6,6,0,},
                          {0,10,10,0,27,28,28,43,0,27,27,27,27,27,0,6,6,6,6,7,14,27,27,27,24,27,0,6,6,0,},
                          {0,14,14,0,28,27,27,44,0,28,27,27,27,43,0,6,6,6,6,6,0,27,27,27,24,27,0,6,6,0,},
                          {0,27,28,0,27,28,28,27,0,27,27,27,28,44,0,0,0,0,0,0,0,0,0,0,24,0,0,10,10,0,},
                          {0,28,27,0,0,0,27,0,0,0,0,0,27,0,0,0,0,0,0,0,0,0,0,0,24,0,0,14,14,0,},
                          {0,28,28,27,42,27,27,27,42,27,27,27,27,27,27,27,42,27,27,28,27,27,27,27,27,27,27,27,27,0,},
                          {0,27,28,27,14,27,27,27,14,27,27,27,27,27,27,27,28,27,27,27,27,27,27,27,27,27,27,27,27,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}};
            tilemap = a3;
            solidInt = 28;
        }
        if(a==4){
            offx = 560;
            offy = -80;
            int[][] a4 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,41,38,16,0,47,38,41,16,38,42,12,38,12,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,10,38,10,0,10,38,10,10,38,10,10,38,10,0,31,32,22,23,33,0,0,},
                          {0,0,0,0,0,0,0,0,0,10,40,10,0,12,40,10,10,40,10,10,40,10,0,18,22,22,22,22,0,0,},
                          {0,0,35,35,36,35,0,0,0,10,10,10,0,10,10,10,10,10,11,10,10,10,0,18,23,22,23,22,0,0,0,},
                          {0,0,22,23,23,22,0,0,0,10,10,15,0,10,10,12,0,0,0,0,0,0,0,18,22,22,22,22,0,},
                          {0,0,22,22,22,22,0,0,0,47,10,10,0,17,10,17,0,1,1,1,8,7,0,18,22,22,22,23,0,},
                          {0,0,22,23,22,22,0,0,0,0,10,0,0,0,10,0,0,1,1,1,1,1,0,18,22,23,22,22,0,0,},
                          {0,0,22,22,22,22,0,0,0,1,1,1,1,9,1,1,1,8,8,27,28,1,0,18,22,23,23,22,0,},
                          {0,0,22,23,23,22,22,19,1,8,8,8,1,3,1,1,8,1,1,24,1,8,0,18,22,22,22,22,0,},
                          {0,0,22,22,22,22,22,19,1,1,9,1,5,8,8,8,1,1,4,26,1,1,0,35,35,37,22,22,0,},
                          {0,0,0,22,22,0,0,0,0,1,1,1,1,1,1,1,1,1,1,24,1,1,0,0,0,0,0,22,0,0,0,0,},
                          {0,0,0,23,23,0,0,0,0,0,10,0,0,0,10,0,0,1,1,24,1,1,22,23,22,22,23,22,0,},
                          {0,0,0,22,22,0,0,0,0,10,12,11,0,41,10,10,0,8,1,25,6,1,0,0,0,0,},
                          {0,0,23,22,23,22,23,22,0,10,12,10,0,10,12,14,0,1,1,24,1,1,0,0,},
                          {0,0,22,22,22,22,0,22,0,10,12,10,0,10,13,10,0,1,1,0,0,0,0,30,1,1,29,0,},
                          {0,0,22,23,22,22,43,22,0,10,38,10,0,10,38,10,0,1,1,0,0,0,0,1,1,1,1,0,0,},
                          {0,0,22,22,22,22,44,23,0,10,38,10,0,10,38,10,0,9,8,0,0,0,0,30,1,1,29,0,0,},
                          {0,0,22,22,23,22,45,22,0,10,38,42,0,10,38,42,0,8,1,0,0,0,0,1,1,2,1,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,22,22,0,0,0,0,0,0,0,0,0,0,0,0,1,9,1,1,1,1,6,4,1,29,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,22,22,0,0,0,0,0,0,0,0,0,0,0,0,4,1,1,1,1,1,1,3,1,1,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,22,23,0,0,0,0,0,0,0,0,0,0,0,0,0,10,10,21,21,21,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,10,10,21,21,21,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,22,23,23,22,0,0,0,0,0,0,0,0,0,10,10,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,23,23,23,23,22,22,22,0,0,0,0,0,0,0,0,0,10,10,20,20,20,20,1,1,0,0,0,},
                          {0,22,22,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,10,10,20,20,20,20,1,1,0,0,0,0,0,0,0,},
                          {0,22,23,23,22,26,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,46,46,22,22,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}};
            tilemap = a4;
            solidInt = 23;
        }
        if(a==5){
            offx = -840;
            offy = 0;
            int[][] a5 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,22,22,0,0,0,0,0,0,0,0,23,23,0,23,23,22,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,22,22,23,22,23,23,23,23,23,23,44,23,23,22,0,0,0,0,0,0,},
                          {0,22,22,23,22,22,22,22,23,23,23,23,23,23,23,43,23,23,22,0,0,0,0,0,0,0,0,},
                          {0,22,22,23,22,0,0,0,0,0,0,0,0,23,23,44,23,23,22,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,26,0,23,23,23,23,23,0,23,23,45,23,23,22,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,24,0,22,22,22,22,22,0,23,23,0,23,23,22,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,46,46,22,22,24,0,22,22,23,23,22,0,23,23,22,23,23,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,22,22,22,22,22,0,23,23,0,23,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,31,32,22,22,22,22,22,22,0,0,0,0,0,0,0,22,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,22,23,23,22,0,16,39,10,39,10,0,22,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,23,22,22,22,22,22,22,0,10,39,10,39,10,0,0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,23,22,0,0,0,0,23,22,0,10,40,10,40,42,0,6,8,8,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,0,31,32,22,23,22,0,10,10,10,10,10,0,8,8,9,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,23,22,0,18,22,23,22,22,0,10,10,10,10,10,0,9,9,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,0,32,22,23,22,22,0,10,10,11,10,10,0,8,9,8,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,23,22,22,22,22,22,22,0,41,10,10,10,10,10,8,8,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,23,22,23,22,0,17,17,10,17,17,0,8,8,9,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,23,23,22,22,22,22,0,0,0,0,0,0,0,8,8,8,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,22,22,22,22,22,22,22,22,0,10,39,11,39,10,0,9,8,8,0,0,0,0,0,0,0,0,0,},
                          {0,0,22,0,0,0,0,0,22,0,10,39,12,39,10,0,8,8,8,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,0,23,23,23,0,22,0,42,40,12,40,12,0,8,9,8,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,23,23,0,23,0,22,0,10,15,10,15,10,0,8,8,8,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,0,0,0,23,0,22,0,12,10,10,10,10,0,8,8,8,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,0,23,23,23,0,22,0,10,10,10,10,16,0,4,9,9,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,0,23,0,0,0,22,22,12,10,14,10,10,10,6,8,8,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,23,23,23,23,0,0,0,17,17,10,17,17,0,3,8,8,0,0,0,0,0,0,0,0,},
                          {0,0,23,0,0,0,23,0,0,0,0,0,0,0,0,0,9,9,8,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,23,23,0,23,0,0,0,0,0,0,0,0,0,8,9,9,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,23,0,23,22,23,22,22,22,22,22,22,0,8,8,4,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,22,22,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,22,0,22,23,23,23,23,23,0,22,22,22,22,22,22,22,22,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,22,0,0,0,0,0,0,0,0,23,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,22,23,23,23,23,23,23,23,23,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},};

            tilemap = a5;
            solidInt = 23;
        }
        if(a==6){
            int a6[][] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,30,9,9,1,8,4,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,30,9,9,27,28,8,0,0,0,0,},
                          {0,22,22,22,22,22,22,22,22,9,9,4,26,8,8,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,30,8,9,25,3,5,0,0,0,0,0,0,},
                          {0,23,23,23,23,23,23,23,0,30,9,9,0,0,0,0,0,0,},
                          {0,23,23,23,23,23,23,23,23,9,9,9,9,9,29,0,0,},
                          {0,23,23,23,23,23,23,23,0,30,8,9,8,9,29,0,0,},
                          {0,46,46,22,22,22,46,46,0,0,0,0,4,9,9,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,9,9,9,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,9,8,9,0,0,0,0,0,},
                          {0,9,8,9,9,9,7,9,9,8,9,3,9,9,9,0,0,0,0,0,0,0,},
                          {0,9,9,9,9,8,9,9,9,9,9,9,9,9,29,0,0,0,0,0,0,0,},
                          {0,9,9,5,9,9,9,8,9,9,6,9,9,9,9,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,8,8,8,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,8,8,8,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,8,8,8,0,0,0,0,0,0,0,},
                          {0,0,23,23,23,20,20,20,20,20,20,0,0,23,0,0,},
                          {0,0,23,23,23,20,20,20,20,20,20,0,0,23,0,0,0,0,0,},
                          {0,0,23,23,0,0,0,0,0,0,0,0,0,23,0,0,0,0,0,0,0,0,0,},
                          {0,0,23,23,23,21,21,21,21,21,23,23,23,23,23,0,0,0,},
                          {0,0,23,23,23,21,21,21,21,21,23,23,23,23,23,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},};
            tilemap = a6;
            solidInt = 23;
            offx = -1120;
            offy = -440;
        }
        if(a==7){
            int[][] a7 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,14,14,0,0,0,0,0,14,14,14,20,20,20,20,20,20,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,45,0,14,14,14,20,20,20,20,20,20,1,1,0,0,0,0,0,0,17,17,17,17,17,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,44,0,14,14,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,17,35,17,35,17,43,0,0,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,44,0,14,14,14,14,0,0,0,0,0,1,1,0,17,17,17,17,17,17,36,17,36,17,43,0,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,44,0,14,14,14,14,0,1,1,1,1,1,1,0,17,17,17,17,17,17,36,17,36,17,43,0,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,0,0,0,0,0,0,0,1,1,1,1,1,1,0,17,17,0,0,0,17,37,17,37,17,43,0,0,0,0,0,0,0,0,},
                          {0,14,14,14,46,14,0,0,0,0,0,0,0,1,1,0,0,0,0,0,1,1,0,0,0,17,35,17,35,17,43,0,0,0,0,0,0,0,0,},
                          {0,14,14,0,0,0,0,0,0,0,1,1,24,1,1,30,30,30,2,1,1,1,43,0,0,17,36,17,36,17,43,0,0,0,0,0,0,0,},
                          {0,14,14,0,31,38,31,1,31,0,1,1,24,1,1,1,1,1,3,1,1,9,43,0,0,17,36,17,36,17,43,0,0,0,0,0,0,0,0,},
                          {0,14,14,0,32,1,32,1,32,0,1,1,24,1,1,1,1,1,5,1,1,1,43,0,0,17,37,17,37,17,43,0,0,0,0,0,0,0,},
                          {0,1,1,0,1,1,7,1,1,0,1,1,24,1,1,29,29,29,29,1,1,1,43,0,0,17,17,18,17,17,0,0,0,0,0,0,0,},
                          {0,1,1,0,0,1,6,0,0,0,1,28,27,1,1,30,30,30,30,1,10,1,43,0,0,17,0,0,0,0,0,0,0,0,0,0,},
                          {0,1,1,1,1,1,5,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,43,0,0,15,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,1,1,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,4,1,4,43,0,0,15,15,15,15,47,0,0,0,0,0,0,0,},
                          {0,1,1,1,1,1,6,1,1,0,1,1,1,1,1,29,29,29,29,1,1,1,43,0,0,15,15,15,15,15,0,0,0,0,0,0,},
                          {0,1,1,0,0,1,5,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,15,15,15,16,47,0,0,0,0,0,0,},
                          {0,1,1,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,15,15,15,15,0,0,0,0,0,},
                          {0,1,1,0,33,1,33,1,33,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,21,22,21,23,21,0,0,0,0,0,0,},
                          {0,1,1,0,34,1,34,38,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,1,12,0,3,31,30,31,1,0,0,1,31,38,31,1,0,0,1,31,30,31,1,0,17,17,17,17,17,17,17,43,0,0,},
                          {0,1,1,0,1,32,1,32,1,1,1,1,32,1,32,1,1,1,1,32,1,32,1,0,17,35,17,35,17,35,17,43,0,0,0,0,0,},
                          {0,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,17,36,17,36,17,36,17,43,0,0,0,0,0,0,},
                          {0,1,1,0,29,1,1,1,1,0,0,29,1,1,1,29,0,0,1,1,1,1,1,0,17,37,17,37,17,37,17,43,0,0,0,0,0,},
                          {0,1,1,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,7,0,0,0,17,17,17,17,18,17,17,43,0,0,0,0,0,},
                          {0,1,1,1,1,1,1,1,1,1,42,40,9,1,1,1,1,39,40,9,6,1,3,0,17,35,17,35,17,35,17,43,0,0,0,0,0,},
                          {0,1,1,1,1,1,1,1,1,4,39,40,1,10,1,1,11,39,41,1,5,1,1,1,17,36,17,36,17,36,17,43,0,0,0,},
                          {0,1,1,1,1,1,1,1,10,42,19,41,8,1,1,1,42,40,10,1,6,1,1,0,17,37,17,37,17,37,17,43,0,0,0,},
                          {0,1,1,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,5,0,0,0,17,17,17,17,17,17,17,43,0,0,0,0,0,0,},
                          {0,12,1,0,30,1,1,1,30,0,0,1,1,1,1,1,0,0,30,1,1,1,30,0,17,35,17,35,17,35,17,43,0,0,0,0,},
                          {0,0,0,0,1,13,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,17,36,17,36,17,36,17,43,0,0,0,0,0,0,},
                          {0,0,0,0,1,33,1,33,1,1,1,1,33,1,33,4,4,4,4,33,1,33,1,0,17,37,17,37,17,37,17,43,0,0,0,0,},
                          {0,0,0,0,38,34,29,34,38,0,0,29,34,29,34,29,0,0,1,34,38,34,1,0,17,17,17,17,17,17,17,43,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}};
            tilemap = a7;
            solidInt = 20;
            offx = -600;
            offy = 240;
        }
        if(a==8){
            int[][] a8 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,14,14,14,14,14,14,0,14,14,45,0,14,14,20,20,20,20,20,14,0,19,0,0,0,0,},
                          {0,14,14,14,14,14,14,14,14,14,14,14,0,14,14,44,0,14,0,0,0,0,0,0,14,0,19,0,0,0,0,0,0,},
                          {0,0,0,0,0,14,0,0,0,0,14,14,14,14,14,44,0,14,0,14,14,14,14,14,14,0,19,0,0,0,0,},
                          {0,19,0,31,1,1,30,1,0,0,14,14,14,14,14,0,0,14,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,},
                          {0,19,0,32,1,1,1,1,0,0,14,14,14,14,14,14,14,14,14,14,14,14,0,19,19,19,19,0,0,0,},
                          {0,19,0,1,1,1,1,1,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,19,0,1,1,1,1,1,0,0,0,19,0,14,0,14,14,14,14,14,14,14,14,14,14,14,0,0,0,0,},
                          {0,19,0,33,1,1,1,1,1,1,0,19,0,14,0,14,14,14,14,14,14,14,14,14,14,14,0,0,0,0,},
                          {0,19,0,34,12,29,29,1,0,1,0,19,0,1,0,14,14,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,},
                          {0,19,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,41,1,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,1,1,1,25,1,1,0,19,19,0,41,1,1,1,1,1,1,1,39,0,0,0,0,0,0,},
                          {0,14,14,14,14,14,14,1,1,1,1,26,1,1,0,19,19,0,1,1,1,1,9,1,1,10,39,0,0,0,},
                          {0,14,14,14,14,0,0,0,1,1,1,24,1,1,0,19,19,0,1,1,1,1,1,1,9,42,19,0,0,0,0,},
                          {0,14,14,14,14,0,0,0,0,1,0,0,0,1,0,19,19,0,1,1,1,0,1,11,42,19,19,0,0,0,},
                          {0,14,14,14,14,14,14,0,1,1,10,0,0,1,0,19,19,0,1,1,1,0,8,1,39,40,39,0,0,0,},
                          {0,14,14,14,14,14,14,0,1,1,1,1,1,1,0,19,19,0,1,1,1,0,1,42,19,41,39,0,0,0,},
                          {0,14,14,14,14,14,14,0,1,1,1,0,0,0,0,0,0,0,1,1,1,0,42,19,41,42,19,0,0,0,},
                          {0,0,0,0,0,0,0,0,1,1,1,0,31,1,30,30,1,0,1,1,1,0,39,40,1,39,19,0,0,0,0,0,},
                          {0,0,1,1,30,7,33,0,1,1,1,0,32,1,1,1,1,0,1,1,1,0,39,41,42,19,19,0,0,0,},
                          {0,0,1,1,1,6,34,0,1,1,1,0,1,1,1,1,1,0,1,1,1,0,41,1,39,19,0,0,0,0,0,0,},
                          {0,0,1,1,1,5,1,1,1,1,1,0,33,1,1,1,1,0,1,1,1,0,1,1,39,19,0,0,0,0,0,0,},
                          {0,0,1,1,13,5,33,0,1,1,1,0,34,1,1,1,1,0,1,1,1,0,1,1,39,19,0,0,0,0,},
                          {0,0,1,29,29,1,34,0,1,1,1,0,0,0,1,1,0,0,1,1,1,0,1,1,19,19,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,1,1,1,0,31,1,1,1,1,0,1,1,1,0,0,0,19,0,0,0,0,0,0,},
                          {0,0,2,30,31,30,1,0,1,1,1,0,32,1,1,1,1,0,1,1,1,0,19,19,19,0,0,0,0,0,},
                          {0,0,3,1,32,1,1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,0,19,19,19,0,0,0,0,0,0,0,},
                          {0,0,1,1,1,4,4,1,4,1,1,0,33,1,1,1,1,0,1,1,1,0,19,19,19,0,0,0,0,0,0,},
                          {0,0,1,1,1,1,1,0,1,1,1,0,34,1,29,29,1,0,1,1,1,0,19,19,19,0,0,0,0,0,},
                          {0,0,1,29,29,29,1,0,1,1,1,0,0,0,0,0,0,0,1,1,1,0,19,19,19,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,19,19,19,0,0,0,0,0,0,},
                          {0,19,19,19,19,19,19,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,19,19,19,0,0,0,0,},
                          {0,19,19,19,19,19,19,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,19,19,19,0,0,0,0,},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},};
            tilemap = a8;
            solidInt = 20;
            offx = 240;
            offy = 560;
  
        }
        if(a==9){
            int[][] a9 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                          {0,1,27,27,1,1,0,7,27,1,1,45,0,1,1,1,1,45,0,0,0,0},
                          {0,1,1,1,1,1,0,7,1,1,1,1,0,1,1,1,1,1,0,0,0,0,0,0,0,},
                          {0,1,1,1,1,29,0,8,1,1,1,1,0,26,1,1,1,45,0,0,0,0,0,},
                          {0,1,1,1,1,1,0,0,0,2,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0},
                          {0,1,1,1,1,1,0,1,1,1,32,1,1,17,1,1,1,1,0,0,0,0,0,0},
                          {0,1,1,1,1,1,0,1,1,1,32,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0},
                          {0,47,1,1,1,1,4,1,1,1,33,1,1,1,1,1,1,1,4,1,1,1,0,0,0,0,},
                          {0,47,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,},
                          {0,6,6,6,6,9,0,1,1,1,34,1,1,26,1,1,1,1,0,0,0,0,0,0,0,0,0,0,},
                          {0,0,0,0,0,0,0,28,1,1,32,1,29,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                          {0,7,1,1,1,35,0,1,1,1,0,0,0,0,35,1,1,32,1,33,1,27,0,0,0,},
                          {0,7,1,1,1,36,0,28,1,1,1,1,1,0,36,29,1,32,1,1,1,1,0,0,0},
                          {0,7,1,1,1,1,0,1,1,1,1,1,1,0,1,1,1,33,1,34,1,1,0,0,0,0},
                          {0,7,1,1,1,1,0,1,1,1,1,1,1,0,1,1,1,1,1,0,1,1,0,0,0,0,0},
                          {0,7,1,1,1,1,4,1,1,1,1,1,1,5,1,1,1,34,1,33,1,1,0,0,0},
                          {0,8,1,1,1,1,0,6,6,6,6,6,6,0,1,16,26,32,1,1,1,26,0,0},
                          {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}};
            tilemap = a9;
            solidInt = 20;
            offx = 160;
            offy = -40; 
        }
        if(a==10){
            int[][] a10 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                           {0,0,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,17,1,1,1,1,0,0,0,0,},
                           {0,0,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,34,1,1,1,1,1,4,1,1,1,1,1,1,0,0,0,0,},
                           {0,0,0,0,0,0,0,2,0,0,0,0,0,0,3,3,0,0,0,0,1,1,26,1,1,0,1,1,1,1,34,1,0,0,0,0,0,},
                           {0,0,0,7,1,1,0,1,0,27,1,27,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,},
                           {0,0,0,8,1,1,1,1,32,1,1,1,0,0,1,1,0,0,0,0,20,20,20,20,20,20,20,20,0,0,0,1,0,0,},
                           {0,0,0,7,1,1,34,1,32,1,1,1,0,0,1,1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,0,0,0,0,},
                           {0,0,0,7,1,1,32,1,33,1,1,35,0,0,1,1,0,0,1,1,1,1,1,1,12,11,11,11,0,1,0,1,0,0,0,},
                           {0,0,0,7,1,1,32,1,1,1,1,36,0,0,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,0,0,0,0,0,},
                           {0,0,0,7,1,10,0,1,0,1,1,1,0,0,1,1,5,1,1,1,1,1,1,1,12,11,11,11,0,1,0,1,0,0,0,0,},
                           {0,0,0,0,0,0,0,3,0,0,0,0,0,0,1,1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,0,0,0,0,0,0,0,0,},
                           {0,0,0,38,1,1,33,1,33,1,1,1,0,0,1,1,0,0,1,0,20,20,20,20,20,20,20,20,0,0,0,1,0,0,0,},
                           {0,1,0,1,1,1,1,1,1,1,1,1,0,0,3,3,0,0,1,0,25,31,31,24,31,31,31,31,0,28,1,1,0,0,0,0,},
                           {0,1,0,39,1,1,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,1,1,1,0,0,0,0,0,0,0,},
                           {0,1,0,0,1,1,1,10,1,1,1,1,0,0,0,0,0,0,0,0,6,6,6,6,9,13,13,1,0,1,1,1,0,0,0,0,0,},
                           {0,1,5,1,1,1,1,0,14,1,1,1,4,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                           {0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,},
                           {0,0,0,38,1,1,1,1,1,1,1,18,0,0,0,0,0,1,0,0,4,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,},
                           {0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,},
                           {0,0,0,39,1,1,34,1,34,1,1,1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,},
                           {0,0,0,0,0,0,0,2,0,0,0,0,0,7,1,1,4,1,5,1,17,1,0,1,1,32,1,0,0,22,1,7,0,0,0,0,},
                           {0,1,1,1,29,0,0,1,0,2,2,0,0,7,1,1,0,1,0,1,1,1,0,1,1,32,1,0,0,22,1,23,0,0,0,0,},
                           {0,35,1,1,29,0,0,1,0,1,1,0,0,8,1,1,0,1,0,1,1,1,0,1,1,32,1,0,0,21,1,22,0,0,0,0,},
                           {0,36,1,1,1,0,0,1,0,1,1,0,0,7,1,1,0,1,0,1,1,1,0,15,1,33,1,0,1,1,1,22,0,0,0,0,0,},
                           {0,1,1,1,1,1,4,1,5,1,1,0,0,0,0,0,0,1,0,0,0,0,0,1,1,1,1,0,1,1,1,22,0,0,0,0,0,0,},
                           {0,0,0,0,0,0,0,0,0,1,1,0,0,4,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,0,21,0,0,0,0,0,0,0,},
                           {0,0,0,1,1,7,1,1,0,1,1,0,0,0,0,0,0,1,0,0,0,0,0,0,1,1,0,0,1,1,1,1,0,0,0,0,0,0,0,},
                           {0,0,0,1,0,23,0,14,0,1,1,0,31,24,31,0,0,3,0,0,0,0,0,0,3,3,0,0,1,1,1,1,0,0,0,0,},
                           {0,0,0,1,1,22,28,1,0,1,1,0,1,26,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,26,0,0,0,0,},
                           {0,0,0,1,1,22,1,1,4,1,1,5,1,1,1,0,1,1,1,32,1,17,27,33,1,1,1,0,2,2,0,0,0,0,0,},
                           {0,0,0,1,1,22,1,1,0,1,1,0,1,1,1,0,1,1,1,32,1,1,1,1,1,1,1,0,1,1,0,0,0,0,0,0,},
                           {0,0,0,1,1,22,1,1,0,1,1,0,1,1,1,0,1,1,1,33,1,1,1,1,1,1,35,0,1,1,0,0,0,0,0,0,},
                           {0,0,0,1,0,21,1,10,0,3,3,0,39,1,1,0,28,1,1,1,1,1,1,1,1,1,36,0,1,1,0,0,0,0,0,0,0,},
                           {0,0,0,1,0,0,0,0,0,0,0,0,38,1,29,0,1,1,1,34,1,1,1,1,1,1,1,0,1,1,0,0,0,0,0,0,0,0,},
                           {0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,32,1,1,1,34,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,},
                           {0,0,0,1,32,1,1,0,0,0,1,1,1,1,1,1,1,1,1,0,1,12,11,0,1,1,1,1,1,1,0,0,0,0,0,0,0,},
                           {0,0,0,1,32,1,35,0,0,0,1,1,1,1,1,1,1,1,1,0,1,12,11,0,1,1,1,1,1,1,0,0,0,0,0,0,},
                           {0,0,0,1,32,1,36,0,0,0,1,0,25,31,24,30,1,1,1,33,1,1,1,32,1,1,0,0,1,1,0,0,0,0,},
                           {0,0,0,1,33,1,1,0,1,33,1,33,26,1,1,32,1,1,1,1,1,1,1,32,1,1,37,0,1,1,0,0,0,0,0,},
                           {0,0,0,1,1,1,1,0,1,1,1,1,1,1,1,32,35,1,1,1,1,1,1,33,1,1,38,0,1,1,0,0,0,0,0,0,},
                           {0,0,0,1,34,1,26,0,1,1,1,1,1,1,1,32,36,29,1,1,1,1,1,1,1,1,1,0,3,3,0,0,0,0,0,},
                           {0,0,0,1,0,0,0,0,1,1,1,1,1,1,1,32,1,1,1,34,1,16,1,34,26,1,39,0,0,0,0,0,0,0,},
                           {0,0,0,1,0,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                           {0,0,0,1,1,1,1,5,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                           {0,0,0,0,0,0,0,0,1,26,1,1,13,13,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                           {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},};
            tilemap = a10;
            solidInt = 19;
            offx = 160;
            offy = -40;      
        }
        if(a==11){
            int[][] a11 = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
                           {0,1,1,0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,0,0,},
                           {0,1,1,0,1,1,1,16,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,},
                           {0,1,1,1,1,1,1,0,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,0,},
                           {0,1,1,1,1,1,1,0,2,2,0,0,0,0,1,1,1,0,1,1,1,1,1,0,},
                           {0,1,1,0,1,1,1,0,1,1,0,1,1,0,1,1,1,0,1,1,1,12,11,0,},
                           {0,1,1,0,25,0,1,0,1,1,1,1,1,0,3,0,0,0,1,1,1,1,1,0,},
                           {0,1,1,0,1,1,1,0,1,1,0,1,1,17,1,1,1,0,1,1,1,1,12,0,},
                           {0,1,1,0,1,1,1,0,1,1,32,1,1,1,1,1,29,0,1,1,1,1,1,0,},
                           {0,1,1,0,0,0,0,0,1,1,32,1,1,1,1,1,1,0,1,1,1,1,1,0,},
                           {0,1,1,0,17,1,1,0,1,1,32,1,1,1,1,1,29,0,1,1,1,1,12,0,},
                           {0,1,1,4,1,1,1,0,1,1,32,1,1,1,1,1,1,0,1,1,1,1,1,0,},
                           {0,1,1,0,1,1,1,0,1,1,0,1,10,1,1,1,1,0,1,1,1,12,11,0,},
                           {0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,1,1,1,1,1,0,},
                           {0,1,1,0,23,1,1,1,1,1,1,32,1,1,0,1,0,0,1,1,1,13,1,0,},
                           {0,1,1,0,22,1,1,1,1,1,1,33,1,1,0,1,0,0,1,39,26,0,0,0,},
                           {0,1,1,0,21,1,1,1,1,1,1,1,1,1,0,1,1,1,0,0,},
                           {0,1,1,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,},
                           {0,1,1,5,1,1,1,1,1,1,1,1,39,1,4,1,1,1,1,1,1,1,0,},
                           {0,1,1,5,1,1,1,1,1,1,1,1,38,1,4,1,1,1,1,1,1,1,0,},
                           {0,1,1,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,1,1,0,},
                           {0,1,1,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,1,1,0,},
                           {0,1,1,0,1,26,1,1,1,1,1,1,1,35,0,0,0,0,0,0,1,1,0,},
                           {0,1,1,0,46,1,46,1,1,1,1,1,1,36,0,0,0,0,0,0,1,1,0,},
                           {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},  };

            tilemap = a11;
            solidInt = 19;
            offx = 160;
            offy = -40;      
        }
        if(!offset){
            offx = 0;
            offy = 0;
        }
        //character.move(offx,offy);
        List actors = getObjects(ActionActor.class);
        for(int i=0;i<actors.size();i++){
            if(!((ActionActor)actors.get(i)).keep()){
                removeObject((Actor)actors.get(i));
            }else{
                ((ActionActor)actors.get(i)).move(offx,offy);
            }
            
        }
        for(int i=0;i<actors.size();i++){
            ((ActionActor)actors.get(i)).keep = false;
        } 
        
        removeObjects(getObjects(Trigger.class));
        //declaredObjects.clear();
        //objectEquates.clear();
        
        for(int i=0;i<declaredObjects.size();i++){
            if( ((Actor)declaredObjects.get(i)).getWorld()==null){
                declaredObjects.remove(i);
                objectEquates.remove(i);   
            }  
        }
        

        tileset = new GreenfootImage("Level"+levelMapOffset[a]+".bmp");
        int tempi = 0;
        GreenfootImage temp = new GreenfootImage(40,40);
        for(int i=0;i<3;i++){
            for(int j=0;j<24;j++){
                temp.clear();
                temp.drawImage(tileset,-j*41,-i*41);
                tiles[tempi] = new GreenfootImage(temp);
                //tiles[tempi].scale(tileWidth,tileWidth);
                tempi++;
            }
        }
        tilesolid = new int[tilemap.length][tilemap[0].length];
        for(int i=0;i<tilesolid.length;i++){
            int[] row = new int[tilemap[i].length];
            tilesolid[i] = row;
            for(int j=0;j<tilesolid[i].length;j++){
                if(tilemap[i][j]==0  ||  tilemap[i][j]>solidInt){
                    tilesolid[i][j] = 1;
                }
            }
        }
        addObject(new EventScript("Area"+a+"/GodScript.txt",this),0,0);
        tilex += offx;
        tiley += offy;
    }
    
    public void setDarkness(int f,int d,double s){
        //System.out.println(f + ":"+d+"+:"+s);
        dest[f] = d;
        speed[f] = (dest[f]-darkness[f])/s;
    }
    
    public void setLightShadow(int a,int b){
        lightx = a;
        lighty = b;
    }
    
    public void setLightBeam(int a,int b,double ang){
        lightShadow.setBeam(a-(int)tilex,b-(int)tiley,ang);
    }
    
    public void turnOnShadows(){
        castShadows = true;
    }
    
    public void turnOffShadows(){
        castShadows = false;
    }
    
    public void setShake(int s){
        shake = s;
    }
    
    
    public double distanceToPlayer(double a,double b){
        return Math.sqrt((tilex+200-a)*(tilex+200-a)+(tiley+200-b)*(tiley+200-b));
    }
    
    public double distanceToPlayer(ActionActor a){
        return Math.sqrt((tilex+200-a.myX)*(tilex+200-a.myX)+(tiley+200-a.myY)*(tiley+200-a.myY));
    }
    
    
    public boolean canSeePlayer(ActionActor a){
        return canSeePlayer(a.myX,a.myY);
    }
    
    public boolean canSeePlayer(ActionActor a,double width){
        return canSeePlayer(a.myX,a.myY,width);
    }
    
    public boolean canSeePlayer(double x1,double y1){
        double dist = distanceToPlayer(x1,y1);
        double dx = ((tilex+200)-x1)*4/dist;
        double dy = ((tiley+200)-y1)*4/dist;
        boolean hit = false;  
        while(dist>0 && !hit){
            x1 += dx;
            y1 += dy;
            hit = tileSolid(x1,y1);
            dist -= 4;
        }  
        return !hit;
    }
    
    public boolean canSeePlayer(double x1,double y1,double width){
        double dist = distanceToPlayer(x1,y1);
        double dx = ((tilex+200)-x1)*4/dist;
        double dy = ((tiley+200)-y1)*4/dist;
        boolean hit = false;  
        while(dist>0 && !hit){
            x1 += dx;
            y1 += dy;
            hit = tileSolid(x1+width,y1+width) || 
                  tileSolid(x1-width,y1+width) || 
                  tileSolid(x1+width,y1-width) || 
                  tileSolid(x1-width,y1-width);
            dist -= 4;
        }  
        return !hit;
    }
    
    public double[] castRay(double ang,double x,double y,int width){
        return castRay(Math.cos(ang),Math.sin(ang),x,y,width);
    }
    
    public double[] castRay(double vx,double vy,double x,double y,int width){
        while(!tileSolid(x+width,y+width) && !tileSolid(x+width,y-width) && !tileSolid(x-width,y+width) && !tileSolid(x-width,y-width)){
            x += vx*3;
            y += vy*3;
        }    
        double[] ret = {x,y};
        return ret;
    }
    
    public double mouseAngle(){
        return mouseAng;
    }
    
    public boolean flashlightHasLight(){
        return lightShadow.hasLight();
    }
    
    public int getHealth(){
        return character.health;
    }
    
    public void drawLine(double x1,double y1,double x2,double y2,int thick){
        getBackground().setColor(Color.black);
        for(int i=0;i<thick;i++){
            for(int j=0;j<thick;j++){
                getBackground().drawLine((int)(x1+i-thick/2-tilex),
                                         (int)(y1+j-thick/2-tiley),
                                         (int)(x2+i-thick/2-tilex),
                                         (int)(y2+j-thick/2-tiley));
            }
        }
    }
    
    public double sin(double x){
        return Math.sin(x);
    }
    
    public double cos(double x){
        return Math.cos(x);
    }
    
    Slider shadowSlider;
    Slider curveSlider;
    Slider particleSlider;
    Slider musicSlider;
    Button backButton;
    Checkbox trigBox;
    TextEntry textEntry;
    
    public void loadMenu(){
        //image = new FastImage(getBackground());
        //back = new GreenfootImage("NH.bmp");
        //NH = new FastImage(back);
        //inMenu = true;
        Greenfoot.setWorld(myMenu);
        myMenu.loadMenu();
        resetCursor();
        //worldBackup = new WorldBackup(this);
        //addObject(menuLoop,0,0);
    }
    
    public void exitMenu(){
        enableCustomCursor();
    }
    
    public void enableCustomCursor(){    
        GreenfootImage image = new GreenfootImage("empty.png"); 
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Point cursorHotSpot = new Point(12,12);
        customCursor = toolkit.createCustomCursor(image.getAwtImage(), cursorHotSpot, "Cursor");
        //panel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        addObject(mouseCursor,0,0);
        panel.setCursor(customCursor);
    }
    
    public void resetCursor(){
        GreenfootImage image = new GreenfootImage("empty.png"); 
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Point cursorHotSpot = new Point(12,12);
        customCursor = toolkit.createCustomCursor(image.getAwtImage(), cursorHotSpot, "Cursor");
        panel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        removeObject(mouseCursor);
    }
    
    public void displayLevelName(){
        for(int i=0;i<levelAreaOffset.length;i++){
            if(levelAreaOffset[i] == areaNumber){
                addObject(new TextMessage("CHAPTER "+(i+1)+":",0),200,50);
                addObject(new TextMessage(levelNames[i],100),200,80);
            }
        }
    }
    
    public boolean isDark(double a,double b){
        a -= tilex;
        b -= tiley;
        if(a<0 || a>399 || b<0 || b>399){
            return true;
        }
        return (shadowBuffer1.getColorAt((int)a,(int)b).equals(Color.black) && shadowBuffer1.getTransparency()==255) ||  
               (lightShadow.getImage().getColorAt((int)a,(int)b).equals(Color.black) && lightShadow.getImage().getTransparency()==255);
    }
    
    public void addActionActor(ActionActor actor,double x,double y){
        actor.myX = x;
        actor.myY = y;
        super.addObject(actor,(int)(x-tilex),(int)(y-tiley));
    }
    
    public void addObject(ActionActor actor,double x,double y){
        actor.myX = x;
        actor.myY = y;
        super.addObject(actor,(int)(x-tilex),(int)(y-tiley));
    }
    
    public void decreaseQuality(){
        if(shadowQuality!=0 && autoQualityAdjust){
            shadowQuality--;
        }
    }
    
    public void previewWeapon(int weaponType){
        weaponPreview.setImage(new GreenfootImage("Items/" + weaponNames[weaponType-1] + ".bmp"));
        weaponPreview.getImage().setTransparency(254);
        weaponPreview.clearImage();
    }
    
}


class ScrollingListener implements MouseWheelListener
{
    int amount = 0;
    
    public void mouseWheelMoved(MouseWheelEvent e){
        amount += e.getWheelRotation();  
        e.consume();
    }
    
    public int getScroll(){
        int t = amount;
        amount = 0;
        return t;
    }
}
